@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@onelab.info>.
@@ -55,6 +55,15 @@
 #include <Python.h>
 #endif
 
+#if defined(HAVE_HPDDM)
+#define MUMPSSUB // direct mumps solver for subdomain solves
+#define DMUMPS // direct mumps solver for the coarse solves
+#define HPDDM_NUMBERING 'C' // 0-based numering
+#define HPDDM_NO_REGEX
+#undef NONE
+#include <HPDDM.hpp>
+#endif
+
 int Message::_commRank = 0;
 int Message::_commSize = 1;
 int Message::_isCommWorld = 1; // is the communicator set to WORLD (==1) or SELF (!=1)
@@ -100,6 +109,7 @@ static void gslErrorHandler(const char *reason, const char *file, int line,
 
 void Message::Initialize(int argc, char **argv)
 {
+  SetNumThreads(1);
   _startTime = GetTimeOfDay();
   _errorCount = 0;
 #if defined(HAVE_PETSC)
@@ -126,21 +136,29 @@ void Message::Initialize(int argc, char **argv)
   std::vector<wchar_t*> wargv(argc ? argc : 1);
   for(int i = 0; i < argc; i++)
     wargv[i] = Py_DecodeLocale(argv[i], NULL);
-  Py_SetProgramName(wargv[0], NULL);
+  Py_SetProgramName(wargv[0]);
   Py_InitializeEx(0);
   PySys_SetArgv(argc, &wargv[0]);
 #endif
 #endif
+#if defined(HAVE_HPDDM)
+  HPDDM::Option& opt = *HPDDM::Option::get();
+  opt.parse(argc, argv, _commRank == 0);
+#endif
 }
 
 void Message::Finalize()
 {
 #if defined(HAVE_PETSC)
-  int initialized, finalized;
-  MPI_Initialized(&initialized);
-  MPI_Finalized(&finalized);
-  if(initialized && !finalized)
-    MPI_Finalize();
+  // if: to avoid a 'PETSC ERROR' message when calling MPI_Finalize()
+  // (not coherent with prior MPI_Init(&argc, &argv)... but do nothing when _commSize == 1)
+  if(_commSize > 1){
+    int initialized, finalized;
+    MPI_Initialized(&initialized);
+    MPI_Finalized(&finalized);
+    if(initialized && !finalized)
+      MPI_Finalize();
+  }
 #endif
   FinalizeSocket();
   FinalizeOnelab();
@@ -321,7 +339,7 @@ void Message::Info(int level, const char *fmt, ...)
   va_end(args);
 
   if(_infoCpu){
-    Cpu(level, false, true, true, true, str);
+    Cpu(level, false, true, true, true, true, str);
     return;
   }
 
@@ -454,11 +472,11 @@ void Message::Cpu(const char *fmt, ...)
   vsnprintf(str, sizeof(str), fmt, args);
   va_end(args);
 
-  Cpu(5, false, true, true, true, str);
+  Cpu(5, false, true, true, true, true, str);
 }
 
 void Message::Cpu(int level, bool printDate, bool printWallTime, bool printCpu,
-                  bool printMem, const char *fmt, ...)
+                  bool printMem, bool printTraffic, const char *fmt, ...)
 {
   if(_verbosity < abs(level)) return;
 
@@ -480,6 +498,18 @@ void Message::Cpu(int level, bool printDate, bool printWallTime, bool printCpu,
 
   if(_commRank && _isCommWorld && level > 0) return;
 
+  if(!mem) printMem = false;
+
+  onelab::remoteNetworkClient *remote = 0;
+  if(_onelabClient){
+    remote = dynamic_cast<onelab::remoteNetworkClient*>(_onelabClient);
+    if(!remote || !remote->getGmshClient())
+      printTraffic = false;
+  }
+  else{
+    printTraffic = false;
+  }
+
   char str[1024];
   va_list args;
   va_start(args, fmt);
@@ -493,7 +523,7 @@ void Message::Cpu(int level, bool printDate, bool printWallTime, bool printCpu,
     time(&now);
     pdate = ctime(&now);
     pdate.resize(pdate.size() - 1);
-    if(printWallTime || printCpu || (printMem && mem))
+    if(printWallTime || printCpu || printMem || printTraffic)
       pdate += ", ";
   }
 
@@ -502,7 +532,7 @@ void Message::Cpu(int level, bool printDate, bool printWallTime, bool printCpu,
     char tmp[128];
     sprintf(tmp, "Wall = %gs", GetWallClockTime());
     pwall = tmp;
-    if(printCpu || (printMem && mem))
+    if(printCpu || printMem || printTraffic)
       pwall += ", ";
   }
 
@@ -514,7 +544,7 @@ void Message::Cpu(int level, bool printDate, bool printWallTime, bool printCpu,
     else
       sprintf(tmp, "CPU = %gs", max[0]);
     pcpu = tmp;
-    if(printMem && mem)
+    if(printMem || printTraffic)
       pcpu += ", ";
   }
 
@@ -526,11 +556,25 @@ void Message::Cpu(int level, bool printDate, bool printWallTime, bool printCpu,
     else
       sprintf(tmp, "Mem = %gMb", max[1]);
     pmem = tmp;
+    if(printTraffic)
+      pmem += ", ";
+  }
+
+  std::string ptraffic = "";
+  if(printTraffic){
+    unsigned long int r = remote->getGmshClient()->ReceivedBytes();
+    unsigned long int s = remote->getGmshClient()->SentBytes();
+    double rmb = (double)r / 1024. / 1024.;
+    double smb = (double)s / 1024. / 1024.;
+    char tmp[128];
+    sprintf(tmp, "Recv/Send = %g/%gMb", rmb, smb);
+    ptraffic = tmp;
   }
 
   char str2[256] = "";
-  if(pdate.size() || pwall.size() || pcpu.size() || pmem.size())
-    sprintf(str2, "(%s%s%s%s)", pdate.c_str(), pwall.c_str(), pcpu.c_str(), pmem.c_str());
+  if(pdate.size() || pwall.size() || pcpu.size() || pmem.size() || ptraffic.size())
+    sprintf(str2, "(%s%s%s%s%s)", pdate.c_str(), pwall.c_str(), pcpu.c_str(),
+            pmem.c_str(), ptraffic.c_str());
   strcat(str, str2);
 
   if(_client){
@@ -764,25 +808,25 @@ void Message::InitializeOnelab(std::string name, std::string sockname)
 }
 
 void Message::AddOnelabNumberChoice(std::string name, const std::vector<double> &value,
-                                    const char *color)
+                                    const char *color, const char *units,
+                                    const char *label, bool visible, bool closed)
 {
   if(_onelabClient){
-    std::vector<double> choices;
     std::vector<onelab::number> ps;
-    _onelabClient->get(ps, name);
-    if(ps.size()){
-      choices = ps[0].getChoices();
-    }
-    else{
+    // optimized exchange (without growing choice vector)
+    _onelabClient->getWithoutChoices(ps, name);
+    if(ps.empty()){
       ps.resize(1);
       ps[0].setName(name);
       ps[0].setReadOnly(true);
     }
     if(color) ps[0].setAttribute("Highlight", color);
+    if(units) ps[0].setAttribute("Units", units);
+    if(label) ps[0].setLabel(label);
+    ps[0].setAttribute("Closed", closed ? "1" : "0");
     ps[0].setValues(value);
-    choices.insert(choices.end(), value.begin(), value.end());
-    ps[0].setChoices(choices);
-    _onelabClient->set(ps[0]);
+    ps[0].setVisible(visible);
+    _onelabClient->setAndAppendChoices(ps[0]);
 
 #if !defined(BUILD_ANDROID) // FIXME: understand why this leads to crashes
     // ask Gmsh to refresh
@@ -794,7 +838,8 @@ void Message::AddOnelabNumberChoice(std::string name, const std::vector<double>
 }
 
 void Message::AddOnelabStringChoice(std::string name, std::string kind,
-                                    std::string value)
+                                    std::string value, bool updateValue,
+                                    bool readOnly)
 {
   if(_onelabClient){
     std::vector<std::string> choices;
@@ -804,15 +849,25 @@ void Message::AddOnelabStringChoice(std::string name, std::string kind,
       choices = ps[0].getChoices();
       if(std::find(choices.begin(), choices.end(), value) == choices.end())
         choices.push_back(value);
+      if(updateValue)
+        ps[0].setValue(value);
     }
     else{
       ps.resize(1);
       ps[0].setName(name);
       ps[0].setKind(kind);
+      ps[0].setValue(value);
       choices.push_back(value);
     }
-    ps[0].setValue(value);
     ps[0].setChoices(choices);
+    if(readOnly){
+      ps[0].setReadOnly(true);
+      ps[0].setAttribute("AutoCheck", "0");
+    }
+    else{
+      ps[0].setReadOnly(false);
+      ps[0].setAttribute("AutoCheck", "1");
+    }
     _onelabClient->set(ps[0]);
   }
 }
@@ -929,6 +984,8 @@ static void _setStandardOptions(onelab::parameter *p, Message::fmap &fopt,
   if(copt.count("Highlight")) p->setAttribute("Highlight", copt["Highlight"][0]);
   if(copt.count("Macro")) p->setAttribute("Macro", copt["Macro"][0]);
   if(copt.count("GmshOption")) p->setAttribute("GmshOption", copt["GmshOption"][0]);
+  if(copt.count("ServerAction")) p->setAttribute("ServerAction", copt["ServerAction"][0]);
+  if(copt.count("Units")) p->setAttribute("Units", copt["Units"][0]);
   if(copt.count("AutoCheck")) // for backward compatibility
     p->setAttribute("AutoCheck", copt["AutoCheck"][0]);
 
@@ -965,7 +1022,9 @@ void Message::ExchangeOnelabParameter(Constant *c, fmap &fopt, cmap &copt)
     bool noRange = true, noChoices = true, noLoop = true;
     bool noGraph = true, noClosed = true;
     if(ps.size()){
-      if(fopt.count("ReadOnly") && fopt["ReadOnly"][0]){ // use local value
+      bool useLocalValue = ps[0].getReadOnly();
+      if(fopt.count("ReadOnly")) useLocalValue = fopt["ReadOnly"][0];
+      if(useLocalValue){
         if(c->Type == VAR_FLOAT){
           ps[0].setValue(c->Value.Float);
         }
@@ -1081,7 +1140,9 @@ void Message::ExchangeOnelabParameter(Constant *c, fmap &fopt, cmap &copt)
     _onelabClient->get(ps, name);
     bool noClosed = true, noMultipleSelection = true;
     if(ps.size()){
-      if(fopt.count("ReadOnly") && fopt["ReadOnly"][0])
+      bool useLocalValue = ps[0].getReadOnly();
+      if(fopt.count("ReadOnly")) useLocalValue = fopt["ReadOnly"][0];
+      if(useLocalValue)
         ps[0].setValue(c->Value.Char); // use local value
       else
 	c->Value.Char = strSave(ps[0].getValue().c_str()); // use value from server
@@ -1193,12 +1254,14 @@ void Message::Barrier()
 #include <omp.h>
 
 int Message::GetNumThreads(){ return omp_get_num_threads(); }
+void Message::SetNumThreads(int num){ omp_set_num_threads(num); }
 int Message::GetMaxThreads(){ return omp_get_max_threads(); }
 int Message::GetThreadNum(){ return omp_get_thread_num(); }
 
 #else
 
 int Message::GetNumThreads(){ return 1; }
+void Message::SetNumThreads(int num){ }
 int Message::GetMaxThreads(){ return 1; }
 int Message::GetThreadNum(){ return 0; }
 
