@@ -39,6 +39,11 @@
 #include <sstream>
 #include "GmshSocket.h"
 
+#undef HAVE_PICOJSON
+#if defined(HAVE_PICOJSON)
+#include "picojson.h"
+#endif
+
 namespace onelab{
 
   // The base parameter class.
@@ -137,7 +142,13 @@ namespace onelab{
     }
     std::string getShortName() const
     {
-      if(_label.size()) return _label;
+      std::string units = getAttribute("Units");
+      if(_label.size()){
+        if(units.empty())
+          return _label;
+        else
+          return _label + " [" + units + "]";
+      }
       std::string s = _name;
       // remove path
       std::string::size_type last = _name.find_last_of('/');
@@ -154,7 +165,10 @@ namespace onelab{
       // start'
       while(s.size() && s[0] >= '0' && s[0] <= '9')
         s = s.substr(1);
-      return s;
+      if(units.empty())
+        return s;
+      else
+        return s + " [" + units + "]";
     }
     int getChanged(const std::string &client="") const
     {
@@ -189,7 +203,7 @@ namespace onelab{
     const std::map<std::string, int> &getClients() const { return _clients; }
     static char charSep() { return '\0'; }
     static double maxNumber() { return 1e200; }
-    static std::string version() { return "1.2"; }
+    static std::string version() { return "1.3"; }
     static int defaultChangedValue() { return 31; }
     static std::string getNextToken(const std::string &msg,
                                     std::string::size_type &first,
@@ -313,35 +327,95 @@ namespace onelab{
       }
       return true;
     }
+    std::string sanitizeJSON(const std::string &in) const
+    {
+      // FIXME: replace \n with \\n, \t with \\t, etc.
+      return in;
+    }
     virtual std::string toJSON() const
     {
       std::ostringstream sstream;
       sstream << "\"type\":\"" << getType() << "\""
-              << ", \"version\":\"" << version() << "\""
-              << ", \"name\":\"" << sanitize(getName()) << "\""
-              << ", \"label\":\"" << sanitize(getLabel()) << "\""
-              << ", \"help\":\"" << sanitize(getHelp()) << "\""
-              << ", \"changedValue\":" << getChangedValue() << "\""
+              << ", \"name\":\"" << sanitizeJSON(getName()) << "\"";
+      if(getLabel().size())
+        sstream << ", \"label\":\"" << sanitizeJSON(getLabel()) << "\"";
+      if(getHelp().size())
+        sstream << ", \"help\":\"" << sanitizeJSON(getHelp()) << "\"";
+      sstream << ", \"changedValue\":" << getChangedValue()
               << ", \"visible\":" << (getVisible() ? "true" : "false")
-              << ", \"readOnly\":" << (getReadOnly() ? "true" : "false")
-              << ", \"attributes\":{ ";
-      for(std::map<std::string, std::string>::const_iterator it = _attributes.begin();
-          it != _attributes.end(); it++){
-        if(it != _attributes.begin()) sstream << ", ";
-        sstream << "\"" << sanitize(it->first) << "\":\""
-                << sanitize(it->second) << "\"";
+              << ", \"readOnly\":" << (getReadOnly() ? "true" : "false");
+      if(_attributes.size()){
+        sstream << ", \"attributes\":{ ";
+        for(std::map<std::string, std::string>::const_iterator it = _attributes.begin();
+            it != _attributes.end(); it++){
+          if(it != _attributes.begin()) sstream << ", ";
+          sstream << "\"" << sanitizeJSON(it->first) << "\":\""
+                  << sanitizeJSON(it->second) << "\"";
+        }
+        sstream << " }";
       }
-      sstream << " }"
-              << ", \"clients\":{ ";
-      for(std::map<std::string, int>::const_iterator it = getClients().begin();
-          it != getClients().end(); it++){
-        if(it != getClients().begin()) sstream << ", ";
-        sstream << "\"" << sanitize(it->first) << "\":"
-                << (it->second ? "true" : "false");
+      if(getClients().size()){
+        sstream << ", \"clients\":{ ";
+        for(std::map<std::string, int>::const_iterator it = getClients().begin();
+            it != getClients().end(); it++){
+          if(it != getClients().begin()) sstream << ", ";
+          sstream << "\"" << sanitizeJSON(it->first) << "\":"
+                  << it->second;
+        }
+        sstream << " }";
       }
-      sstream << " }";
       return sstream.str();
     }
+#if defined(HAVE_PICOJSON)
+    virtual bool fromJSON(const picojson::value::object& par)
+    {
+      for(picojson::value::object::const_iterator it = par.begin(); it != par.end(); ++it){
+        if(it->first == "name"){
+          if(!it->second.is<std::string>()) return false;
+          setName(it->second.get<std::string>());
+        }
+        else if(it->first == "label"){
+          if(!it->second.is<std::string>()) return false;
+          setLabel(it->second.get<std::string>());
+        }
+        else if(it->first == "help"){
+          if(!it->second.is<std::string>()) return false;
+          setHelp(it->second.get<std::string>());
+        }
+        else if(it->first == "changedValue"){
+          if(!it->second.is<double>()) return false;
+          setChangedValue((int)it->second.get<double>());
+        }
+        else if(it->first == "visible"){
+          if(!it->second.is<bool>()) return false;
+          setVisible(it->second.get<bool>());
+        }
+        else if(it->first == "readOnly"){
+          if(!it->second.is<bool>()) return false;
+          setReadOnly(it->second.get<bool>());
+        }
+        else if(it->first == "attributes"){
+          if(!it->second.is<picojson::object>()) return false;
+          const picojson::value::object &obj = it->second.get<picojson::object>();
+          for (picojson::value::object::const_iterator i = obj.begin(); i != obj.end(); ++i) {
+            std::string key(i->first);
+            if(!i->second.is<std::string>()) return false;
+            setAttribute(key, i->second.get<std::string>());
+          }
+        }
+        else if(it->first == "clients"){
+          if(!it->second.is<picojson::object>()) return false;
+          const picojson::value::object &obj = it->second.get<picojson::object>();
+          for (picojson::value::object::const_iterator i = obj.begin(); i != obj.end(); ++i) {
+            std::string client(i->first);
+            if(!i->second.is<double>()) return false;
+            addClient(client, (int)i->second.get<double>());
+          }
+        }
+      }
+      return true;
+    }
+#endif
   };
 
   class parameterLessThan{
@@ -481,7 +555,7 @@ namespace onelab{
       std::ostringstream sstream;
       sstream.precision(16);
       sstream << "{ " << parameter::toJSON()
-              << ", \"values\":[";
+              << ", \"values\":[ ";
       for(unsigned int i = 0; i < _values.size(); i++){
         if(i) sstream << ", ";
         sstream << _values[i];
@@ -490,22 +564,78 @@ namespace onelab{
               << ", \"min\":" << _min
               << ", \"max\":" << _max
               << ", \"step\":" << _step
-              << ", \"index\":" << _index
-              << ", \"choices\":[ ";
-      for(unsigned int i = 0; i < _choices.size(); i++){
-        if(i) sstream << ", ";
-        sstream << _choices[i];
+              << ", \"index\":" << _index;
+      if(_choices.size()){
+        sstream << ", \"choices\":[ ";
+        for(unsigned int i = 0; i < _choices.size(); i++){
+          if(i) sstream << ", ";
+          sstream << _choices[i];
+        }
+        sstream << " ]";
       }
-      sstream << " ]"
-              << ", \"valueLabels\":{ ";
-      for(std::map<double, std::string>::const_iterator it = _valueLabels.begin();
-          it != _valueLabels.end(); it++){
-        if(it != _valueLabels.begin()) sstream << ", ";
-        sstream << "\"" << sanitize(it->second) << "\":" << it->first;
+      if(_valueLabels.size()){
+        sstream << ", \"valueLabels\":{ ";
+        for(std::map<double, std::string>::const_iterator it = _valueLabels.begin();
+            it != _valueLabels.end(); it++){
+          if(it != _valueLabels.begin()) sstream << ", ";
+          sstream << "\"" << sanitizeJSON(it->second) << "\":" << it->first;
+        }
+        sstream << " }";
       }
-      sstream << " } }";
+      sstream << " }";
       return sstream.str();
     }
+#if defined(HAVE_PICOJSON)
+    bool fromJSON(const picojson::value::object& par)
+    {
+      if(!parameter::fromJSON(par)) return false;
+      for(picojson::value::object::const_iterator it = par.begin(); it != par.end(); ++it){
+        if(it->first == "values"){
+          if(!it->second.is<picojson::array>()) return false;
+          const picojson::value::array &arr = it->second.get<picojson::array>();
+          _values.resize(arr.size());
+          for(unsigned int i = 0; i < arr.size(); i++){
+            if(!arr[i].is<double>()) return false;
+            _values[i] = arr[i].get<double>();
+          }
+        }
+        else if(it->first == "min"){
+          if(!it->second.is<double>()) return false;
+          setMin(it->second.get<double>());
+        }
+        else if(it->first == "max"){
+          if(!it->second.is<double>()) return false;
+          setMax(it->second.get<double>());
+        }
+        else if(it->first == "step"){
+          if(!it->second.is<double>()) return false;
+          setStep(it->second.get<double>());
+        }
+        else if(it->first == "index"){
+          if(!it->second.is<double>()) return false;
+          setIndex((int)it->second.get<double>());
+        }
+        else if(it->first == "choices"){
+          if(!it->second.is<picojson::array>()) return false;
+          const picojson::value::array &arr = it->second.get<picojson::array>();
+          _choices.resize(arr.size());
+          for(unsigned int i = 0; i < arr.size(); i++){
+            if(!arr[i].is<double>()) return false;
+            _choices[i] = arr[i].get<double>();
+          }
+        }
+        else if(it->first == "valueLabels"){
+          if(!it->second.is<picojson::object>()) return false;
+          const picojson::value::object &obj = it->second.get<picojson::object>();
+          for (picojson::value::object::const_iterator i = obj.begin(); i != obj.end(); ++i) {
+            if(!i->second.is<double>()) return false;
+            _valueLabels[i->second.get<double>()] = i->first;
+          }
+        }
+      }
+      return true;
+    }
+#endif
   };
 
   // The string class. A string has a mutable "kind", that can be changed at
@@ -589,18 +719,53 @@ namespace onelab{
               << ", \"values\":[ " ;
       for(unsigned int i = 0; i < _values.size(); i++){
         if(i) sstream << ", ";
-        sstream << "\"" << sanitize(_values[i]) << "\"";
+        sstream << "\"" << sanitizeJSON(_values[i]) << "\"";
       }
-      sstream << " ] "
-              << ", \"kind\":\"" << sanitize(_kind) <<  "\""
-              << ", \"choices\":[ ";
-      for(unsigned int i = 0; i < _choices.size(); i++){
-        if(i) sstream << ", ";
-        sstream << "\"" << sanitize(_choices[i]) << "\"";
+      sstream << " ] ";
+      if(_kind.size())
+        sstream << ", \"kind\":\"" << sanitizeJSON(_kind) <<  "\"";
+      if(_choices.size()){
+        sstream << ", \"choices\":[ ";
+        for(unsigned int i = 0; i < _choices.size(); i++){
+          if(i) sstream << ", ";
+          sstream << "\"" << sanitizeJSON(_choices[i]) << "\"";
+        }
+        sstream << " ]";
       }
-      sstream << " ] }";
+      sstream << " }";
       return sstream.str();
     }
+#if defined(HAVE_PICOJSON)
+    bool fromJSON(const picojson::value::object& par)
+    {
+      if(!parameter::fromJSON(par)) return false;
+      for(picojson::value::object::const_iterator it = par.begin(); it != par.end(); ++it){
+        if(it->first == "values"){
+          if(!it->second.is<picojson::array>()) return false;
+          const picojson::value::array &arr = it->second.get<picojson::array>();
+          _values.resize(arr.size());
+          for(unsigned int i = 0; i < arr.size(); i++){
+            if(!arr[i].is<std::string>()) return false;
+            _values[i] = arr[i].get<std::string>();
+          }
+        }
+        else if(it->first == "kind"){
+          if(!it->second.is<std::string>()) return false;
+          setKind(it->second.get<std::string>());
+        }
+        else if(it->first == "choices"){
+          if(!it->second.is<picojson::array>()) return false;
+          const picojson::value::array &arr = it->second.get<picojson::array>();
+          _choices.resize(arr.size());
+          for(unsigned int i = 0; i < arr.size(); i++){
+            if(!arr[i].is<std::string>()) return false;
+            _choices[i] = arr[i].get<std::string>();
+          }
+        }
+      }
+      return true;
+    }
+#endif
   };
 
   // The parameter space, i.e., the set of parameters stored and handled by the
@@ -720,14 +885,24 @@ namespace onelab{
         if(!done) done = _clear(name, client, _strings);
       }
     }
-    bool set(const number &p,
-             const std::string &client=""){ return _set(p, client, _numbers); }
-    bool set(const string &p,
-             const std::string &client=""){ return _set(p, client, _strings); }
+    bool set(const number &p, const std::string &client="")
+    {
+      return _set(p, client, _numbers);
+    }
+    bool set(const string &p, const std::string &client="")
+    {
+      return _set(p, client, _strings);
+    }
     bool get(std::vector<number> &ps, const std::string &name="",
-             const std::string &client=""){ return _get(ps, name, client, _numbers); }
-    bool get(std::vector<onelab::string> &ps, const std::string &name="",
-             const std::string &client=""){ return _get(ps, name, client, _strings); }
+             const std::string &client="")
+    {
+      return _get(ps, name, client, _numbers);
+    }
+    bool get(std::vector<string> &ps, const std::string &name="",
+             const std::string &client="")
+    {
+      return _get(ps, name, client, _strings);
+    }
     void getPtr(number **ptr, const std::string name, const std::string client="")
     {
       *ptr = _getPtr(name, client, _numbers);
@@ -812,17 +987,17 @@ namespace onelab{
         onelab::parameter::getInfoFromChar(msg[i], version, type, name);
         if(onelab::parameter::version() != version) return false;
         if(type == "number"){
-          onelab::number p; p.fromChar(msg[i]); set(p, client);
+          number p; p.fromChar(msg[i]); set(p, client);
         }
         else if(type == "string"){
-          onelab::string p; p.fromChar(msg[i]); set(p, client);
+          string p; p.fromChar(msg[i]); set(p, client);
         }
         else
           return false;
       }
       return true;
     }
-    void toJSON(std::string &json, const std::string &creator="",
+    bool toJSON(std::string &json, const std::string &creator="",
                 const std::string &client="") const
     {
       time_t now;
@@ -833,18 +1008,61 @@ namespace onelab{
       json += "{ \"onelab\":{\n";
       json += "  \"creator\":\"" + creator + "\",\n";
       json += "  \"date\":\"" + t + "\",\n";
-      json += "  \"parameters\":[ \n";
+      json += "  \"version\":\"" + parameter::version() + "\",\n";
+      json += "  \"parameters\":[\n";
       std::set<parameter*, parameterLessThan> ps;
       _getAllParameters(ps);
       for(std::set<parameter*, parameterLessThan>::const_iterator it = ps.begin();
           it != ps.end(); it++){
+        if(it != ps.begin()) json += ",\n";
         if(client.empty() || (*it)->hasClient(client)){
-	  if((*it)->getAttribute("NotInDb") != "True")
-	    json += "    " + (*it)->toJSON() + "\n";
+	  if((*it)->getAttribute("NotInDb") != "True"){
+	    json += "    " + (*it)->toJSON();
+          }
 	}
       }
-      json += "] }\n";
+      json += "\n  ] }\n}\n";
+      return true;
     }
+#if defined(HAVE_PICOJSON)
+    bool fromJSON(const std::string &json, const std::string &client="")
+    {
+      picojson::value v;
+      std::string err = picojson::parse(v, json);
+      if(err.size()) return false;
+      if(!v.is<picojson::object>()) return false;
+      const picojson::value::object &obj = v.get<picojson::object>();
+      for (picojson::value::object::const_iterator i = obj.begin(); i != obj.end(); ++i) {
+        if(i->first == "onelab"){ // onelab database
+          if(!i->second.is<picojson::object>()) return false;
+          const picojson::value::object &db = i->second.get<picojson::object>();
+          for (picojson::value::object::const_iterator j = db.begin(); j != db.end(); ++j) {
+            if(j->first == "version"){
+              if(!j->second.is<std::string>()) return false;
+              if(j->second.get<std::string>() != parameter::version()) return false;
+            }
+            else if(j->first == "parameters"){
+              if(!j->second.is<picojson::array>()) return false;
+              const picojson::value::array &arr = j->second.get<picojson::array>();
+              for(unsigned int k = 0; k < arr.size(); k++){
+                if(!arr[k].is<picojson::object>()) return false;
+                const picojson::value::object &par = arr[k].get<picojson::object>();
+                picojson::value::object::const_iterator it = par.find("type");
+                if(it == par.end()) return false;
+                if(it->second.to_str() == "number"){
+                  number p; p.fromJSON(par); set(p, client);
+                }
+                else if(it->second.to_str() == "string"){
+                  string p; p.fromJSON(par); set(p, client);
+                }
+              }
+            }
+          }
+        }
+      }
+      return true;
+    }
+#endif
   };
 
   // The onelab client: a class that communicates with the onelab server. Each
@@ -881,7 +1099,11 @@ namespace onelab{
     virtual bool set(const number &p) = 0;
     virtual bool set(const string &p) = 0;
     virtual bool get(std::vector<number> &ps, const std::string &name="") = 0;
-    virtual bool get(std::vector<onelab::string> &ps, const std::string &name="") = 0;
+    virtual bool get(std::vector<string> &ps, const std::string &name="") = 0;
+    virtual bool setAndAppendChoices(const number &p) = 0;
+    virtual bool setAndAppendChoices(const string &p) = 0;
+    virtual bool getWithoutChoices(std::vector<number> &ps, const std::string &name="") = 0;
+    virtual bool getWithoutChoices(std::vector<string> &ps, const std::string &name="") = 0;
     std::vector<std::string> toChar()
     {
       std::vector<std::string> out;
@@ -898,10 +1120,10 @@ namespace onelab{
         onelab::parameter::getInfoFromChar(msg[i], version, type, name);
         if(onelab::parameter::version() != version) return false;
         if(type == "number"){
-          onelab::number p; p.fromChar(msg[i]); set(p);
+          number p; p.fromChar(msg[i]); set(p);
         }
         else if(type == "string"){
-          onelab::string p; p.fromChar(msg[i]); set(p);
+          string p; p.fromChar(msg[i]); set(p);
         }
         else
           return false;
@@ -919,6 +1141,7 @@ namespace onelab{
       return false;
     }
   };
+
   // The onelab server: a singleton that stores the parameter space and
   // interacts with onelab clients.
   class server{
@@ -1000,10 +1223,16 @@ namespace onelab{
       if(parameter::fromFile(msg, fp)) return fromChar(msg, client);
       return false;
     }
-    void toJSON(std::string &json, const std::string &client="")
+    bool toJSON(std::string &json, const std::string &client="")
     {
-      _parameterSpace.toJSON(json, client);
+      return _parameterSpace.toJSON(json, client);
     }
+#if defined(HAVE_PICOJSON)
+    bool fromJSON(const std::string &json, const std::string &client="")
+    {
+      return _parameterSpace.fromJSON(json, client);
+    }
+#endif
   };
 
   // A local client, which lives in the same memory space as the server.
@@ -1038,8 +1267,46 @@ namespace onelab{
     virtual bool set(const string &p){ return _set(p); }
     virtual bool get(std::vector<number> &ps,
                      const std::string &name=""){ return _get(ps, name); }
-    virtual bool get(std::vector<onelab::string> &ps,
+    virtual bool get(std::vector<string> &ps,
                      const std::string &name=""){ return _get(ps, name); }
+    virtual bool setAndAppendChoices(const number &p)
+    {
+      std::vector<number> ps;
+      _get(ps, _name);
+      std::vector<double> choices;
+      if(ps.size()) choices = ps[0].getChoices();
+      choices.insert(choices.end(), p.getChoices().begin(), p.getChoices().end());
+      number p2(p);
+      p2.setChoices(choices);
+      return _set(p2);
+    }
+    virtual bool setAndAppendChoices(const string &p)
+    {
+      std::vector<string> ps;
+      _get(ps, _name);
+      std::vector<std::string> choices;
+      if(ps.size()) choices = ps[0].getChoices();
+      choices.insert(choices.end(), p.getChoices().begin(), p.getChoices().end());
+      string p2(p);
+      p2.setChoices(choices);
+      return _set(p2);
+    }
+    virtual bool getWithoutChoices(std::vector<number> &ps,
+                                   const std::string &name="")
+    {
+      bool ret = _get(ps, name);
+      for(unsigned int i = 0; i < ps.size(); i++)
+        ps[i].setChoices(std::vector<double>());
+      return ret;
+    }
+    virtual bool getWithoutChoices(std::vector<string> &ps,
+                                   const std::string &name="")
+    {
+      bool ret = _get(ps, name);
+      for(unsigned int i = 0; i < ps.size(); i++)
+        ps[i].setChoices(std::vector<std::string>());
+      return ret;
+    }
   };
 
   // The local part of a network client.
@@ -1096,21 +1363,26 @@ namespace onelab{
     GmshClient *_gmshClient;
     // number of subclients
     int _numSubClients;
-    template <class T> bool _set(const T &p)
+    template <class T> bool _set(const T &p, bool withChoices=true)
     {
       if(!_gmshClient) return false;
       std::string msg = p.toChar();
-      _gmshClient->SendMessage(GmshSocket::GMSH_PARAMETER, msg.size(), &msg[0]);
+      _gmshClient->SendMessage(withChoices ? GmshSocket::GMSH_PARAMETER :
+                               GmshSocket::GMSH_PARAMETER_WITHOUT_CHOICES,
+                               msg.size(), &msg[0]);
       return true;
     }
-    template <class T> bool _get(std::vector<T> &ps, const std::string &name="")
+    template <class T> bool _get(std::vector<T> &ps, const std::string &name="",
+                                 bool withChoices=true)
     {
       ps.clear();
       if(!_gmshClient) return false;
       T p(name);
       std::string msg = p.toChar();
-      if (name.size())
-	_gmshClient->SendMessage(GmshSocket::GMSH_PARAMETER_QUERY, msg.size(), &msg[0]);
+      if(name.size())
+	_gmshClient->SendMessage(withChoices ? GmshSocket::GMSH_PARAMETER_QUERY :
+                                 GmshSocket::GMSH_PARAMETER_QUERY_WITHOUT_CHOICES,
+                                 msg.size(), &msg[0]);
       else // get all parameters
 	_gmshClient->SendMessage(GmshSocket::GMSH_PARAMETER_QUERY_ALL, msg.size(), &msg[0]);
 
@@ -1228,10 +1500,24 @@ namespace onelab{
     }
     virtual bool set(const number &p){ return _set(p); }
     virtual bool set(const string &p){ return _set(p); }
-    virtual bool get(std::vector<number> &ps,
-                     const std::string &name=""){ return _get(ps, name); }
-    virtual bool get(std::vector<onelab::string> &ps,
-                     const std::string &name=""){ return _get(ps, name); }
+    virtual bool get(std::vector<number> &ps, const std::string &name="")
+    {
+      return _get(ps, name);
+    }
+    virtual bool get(std::vector<string> &ps, const std::string &name="")
+    {
+      return _get(ps, name);
+    }
+    virtual bool setAndAppendChoices(const number &p){ return _set(p, false); }
+    virtual bool setAndAppendChoices(const string &p){ return _set(p, false); }
+    virtual bool getWithoutChoices(std::vector<number> &ps, const std::string &name="")
+    {
+      return _get(ps, name, false);
+    }
+    virtual bool getWithoutChoices(std::vector<string> &ps, const std::string &name="")
+    {
+      return _get(ps, name, false);
+    }
     void sendInfo(const std::string &msg)
     {
       if(_gmshClient) _gmshClient->Info(msg.c_str());
@@ -1262,10 +1548,15 @@ namespace onelab{
     }
     void runNonBlockingSubClient(const std::string &name, const std::string &command)
     {
+#if !defined(BUILD_IOS)
       if(!_gmshClient){
-        system(command.c_str());
+        int res = system(command.c_str());
+        if(res){
+          // report error
+        }
         return;
       }
+#endif
       std::string msg = name + parameter::charSep() + command;
       _gmshClient->SendMessage(GmshSocket::GMSH_CONNECT, msg.size(), &msg[0]);
       _numSubClients += 1;
