@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@onelab.info>.
@@ -6,6 +6,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <string>
+#include <set>
 #include "GetDPConfig.h"
 #include "ProData.h"
 #include "ProDefine.h"
@@ -33,6 +34,13 @@ int fcmp_Integer(const void *a, const void *b)
   return(*(int*)a - *(int*)b );
 }
 
+int fcmp_Integer2(const void *a, const void *b)
+{
+  static int result ;
+  if ( ( result = (*(int*)a - *(int*)b ) ) != 0 )  return result ;
+  return (*((int*)a+1) - *((int*)b+1) );
+}
+
 int fcmp_Constant (const void *a, const void *b)
 {
   return ( strcmp(((struct Constant *)a)->Name, ((struct Constant *)b)->Name));
@@ -200,10 +208,16 @@ void Init_ProblemStructure()
   Current.Breakpoint = 0.0;
   Current.Iteration = 0.0;
   Current.Residual = 0.0;
+  Current.ResidualN = 0.0; //+++
+  Current.Residual_Iter1 = 1.0; //+++
   Current.RelativeDifference = 0.0;
   Current.RelativeDifferenceOld = 0.0;
   Current.RelaxationFactor = 0.0;
-  Current.KSPIts = 0.0;
+  Current.RelaxFac = 0.0; //+++
+  Current.NbrTestedFac = 0.0; //+++
+  Current.KSPIterations = 0.0;
+  Current.KSPIteration = 0.0;
+  Current.KSPResidual = 0.0;
 }
 
 // FIXME: TODO to remove parser memory leaks!
@@ -312,10 +326,12 @@ void Read_ProblemStructure(const char *name)
 
   char AbsPath[4096];
   int i;
+  bool absolute = false;
 
   if((strlen(name) > 0 && (name[0] == '/' || name[0] == '\\')) ||
      (strlen(name) > 3 && name[1] == ':' && (name[2] == '\\' || name[2] == '/'))){
     // name is an absolute path
+    absolute = true;
     strcpy(AbsPath, name);
   }
   else{
@@ -328,12 +344,36 @@ void Read_ProblemStructure(const char *name)
 
   Message::Info("Loading problem definition '%s'", AbsPath);
 
+  Message::AddOnelabStringChoice(Message::GetOnelabClientName() + "/{Input files",
+                                 "file", AbsPath, true, true);
+
   // opening the file in text mode messes up the loops (they use
   // fsetpos/fgetpos) on Windows without Cygwin; not sure why, but
   // opening the file in binary mode fixes the problem
   if(!(getdp_yyin = FOpen(AbsPath, "rb"))){
-    Message::Error("Unable to open file '%s'", AbsPath);
-    return;
+    if(getdp_yyincludenum > 0 && !absolute){
+      // try to find included files in a standard directory
+      Message::Info("File `%s' not found, looking in $GETDP_TEMPLATES", AbsPath);
+      const char* template_dir = getenv("GETDP_TEMPLATES");
+      if(template_dir){
+        strcpy(AbsPath, template_dir);
+        strcat(AbsPath, "/");
+        strcat(AbsPath, name);
+        printf("trying to open %s\n", AbsPath);
+        if(!(getdp_yyin = FOpen(AbsPath, "rb"))){
+          Message::Error("Unable to open file '%s'", AbsPath);
+          return;
+        }
+      }
+      else{
+        Message::Error("No template directory defined");
+        return;
+      }
+    }
+    else{
+      Message::Error("Unable to open file '%s'", AbsPath);
+      return;
+    }
   }
 
   getdp_yyerrorlevel = 0;  getdp_yylinenum = 1; getdp_yyincludenum=0;
@@ -371,9 +411,8 @@ void Finalize_ProblemStructure()
 {
   for(unsigned int i = 0; i < openFiles.size(); i++)
     fclose(openFiles[i]);
+  openFiles.clear();
   MacroManager::Instance()->clear();
-
-  // Here we should parse any ONELAB-defined functions (+ their context)
 }
 
 char *Get_ExpressionName(int Index)
@@ -466,12 +505,14 @@ void Print_WholeQuantity(List_T *WholeQuantity, List_T *DQ_L)
       Message::Check(" %.8g", (WQ+k)->Case.Constant);
       break;
 
-    case WQ_MHTRANSFORM : //****
+    case WQ_MHTRANSFORM :
       Message::Check(" MHTransform[ ");
-      Message::Check("%s",
-                     Get_ExpressionName((WQ+k)->Case.MHTransform.Index));
+      Message::Check("%s", Get_ExpressionName((WQ+k)->Case.MHTransform.Index));
       Message::Check("[");
-      Print_WholeQuantity((WQ+k)->Case.MHTransform.WholeQuantity, DQ_L);
+      for(int i = 0; i < List_Nbr((WQ+k)->Case.MHTransform.WholeQuantity_L); i++){
+        List_T *wq; List_Read((WQ+k)->Case.MHTransform.WholeQuantity_L, i, &wq);
+        Print_WholeQuantity(wq, DQ_L);
+      }
       Message::Check(" ] ]{ %d }", (WQ+k)->Case.MHTransform.NbrPoints);
      break;
 
@@ -479,9 +520,6 @@ void Print_WholeQuantity(List_T *WholeQuantity, List_T *DQ_L)
       Message::Check(" MHJacNL[ ");
       Message::Check("%s",
                      Get_ExpressionName((WQ+k)->Case.MHJacNL.Index));
-      //Message::Check("[");
-      //Print_WholeQuantity((WQ+k)->Case.MHTransform.WholeQuantity, DQ_L);
-      //Message::Check("] ]{ %d, %d}", (WQ+k)->Case.MHJacNL.NbrPoints, (WQ+k)->Case.MHJacNL.FreqOffSet);
       Message::Check("]{ %d, %d}", (WQ+k)->Case.MHJacNL.NbrPoints, (WQ+k)->Case.MHJacNL.FreqOffSet);
       break;
 
@@ -684,6 +722,27 @@ void Print_Expression()
 	Message::Check("  DefineFunction[ %s ];\n", EX->Name);
       break;
 
+    case PIECEWISEFUNCTION2 :
+      for (j = 0;
+	   j < List_Nbr(EX->Case.PieceWiseFunction2.ExpressionPerRegion); j++) {
+        struct ExpressionPerRegion2 *EXPR;
+	EXPR = (struct ExpressionPerRegion2*)
+	  List_Pointer(EX->Case.PieceWiseFunction2.ExpressionPerRegion, j);
+	Message::Check("  %s[%d,%d] = Exp[%s];\n",
+                       EX->Name, EXPR->RegionIndex[0], EXPR->RegionIndex[1],
+                       Get_ExpressionName(EXPR->ExpressionIndex));
+      }
+      if (EX->Case.PieceWiseFunction2.ExpressionIndex_Default >= 0) {
+	Message::
+          Check("  %s[All,All] = Exp[%s];\n",
+                EX->Name,
+                Get_ExpressionName(EX->Case.PieceWiseFunction.ExpressionIndex_Default));
+      }
+      if (!List_Nbr(EX->Case.PieceWiseFunction2.ExpressionPerRegion) &&
+          EX->Case.PieceWiseFunction2.ExpressionIndex_Default < 0)
+	Message::Check("  DefineFunction[ %s ];\n", EX->Name);
+      break;
+
     case UNDEFINED_EXP :
       Message::Check("  DefineFunction[ %s ];\n", EX->Name);
       break;
@@ -1218,6 +1277,10 @@ void Print_Formulation()
                        ((struct Group *)
                         List_Pointer(Problem_S.Group, FE->Case.GlobalTerm.InIndex))->Name );
 
+        if (FE->Case.GlobalTerm.SubType != EQ_ST_SELF)
+          Message::Check("                 SubType %s;\n",
+                         Get_StringForDefine(Equation_SubType, FE->Case.GlobalTerm.SubType));
+
 	Message::Check("      /* Inventaire des DQ (%d) [%d,%d] :",
                        FE->Case.GlobalTerm.Term.NbrQuantityIndex,
                        FE->Case.GlobalTerm.Term.DefineQuantityIndexDof,
@@ -1500,12 +1563,20 @@ void Print_Operation(struct Resolution *RE, List_T *Operation_L)
                       List_Pointer(RE->DefineSystem, OPE->DefineSystemIndex))->Name);
       break;
 
-    case OPERATION_DEFORMEMESH :
-      Message::Check("      DeformeMesh [%s, %s,  '%s']; \n",
-                     ((struct DefineSystem *)
-                      List_Pointer(RE->DefineSystem, OPE->DefineSystemIndex))->Name,
-                     OPE->Case.DeformeMesh.Quantity,
-                     OPE->Case.DeformeMesh.Name_MshFile);
+    case OPERATION_DEFORMMESH :
+      if(OPE->Case.DeformMesh.Quantity && OPE->Case.DeformMesh.Quantity2 &&
+         OPE->Case.DeformMesh.Quantity3)
+        Message::Check("      DeformMesh [%s, {%s, %s, %s}, '%s']; \n",
+                       ((struct DefineSystem *)
+                        List_Pointer(RE->DefineSystem, OPE->DefineSystemIndex))->Name,
+                       OPE->Case.DeformMesh.Quantity, OPE->Case.DeformMesh.Quantity2,
+                       OPE->Case.DeformMesh.Quantity3, OPE->Case.DeformMesh.Name_MshFile);
+      else
+        Message::Check("      DeformMesh [%s, %s, '%s']; \n",
+                       ((struct DefineSystem *)
+                        List_Pointer(RE->DefineSystem, OPE->DefineSystemIndex))->Name,
+                       OPE->Case.DeformMesh.Quantity,
+                       OPE->Case.DeformMesh.Name_MshFile);
       break;
 
     case OPERATION_GMSHREAD :
@@ -2005,7 +2076,7 @@ void Print_ListPostOperation(int choose, int Flag_LPOS, char *name[NBR_MAX_POS])
         charOptions["Choices"] = choices;
         charOptions["Name"].push_back(Message::GetOnelabClientName() + "/2PostOperationChoices");
         charOptions["Label"].push_back("Post-processing");
-        charOptions["MultipleSelection"].push_back("1");
+        charOptions["MultipleSelection"].push_back("0");
         Message::ExchangeOnelabParameter(&c, floatOptions, charOptions);
         if(choose){
           std::string str(c.Value.Char);
