@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@onelab.info>.
@@ -47,13 +47,25 @@
 // - MAX_STACK_SIZE controls the size of the stack used in the evaluation of
 // expressions ; keep this large enough by default to allow for complex expressions
 
+// - MAX_STACK_SIZE0 controls the size of the stack used in the evaluation of expressions;
+// equals 8 by default (tensors), reduced to 2 in multiharmonic case for allowing to go up
+// in the number of harmonics... Put back to 8 if you need tensors!
+
 #if !defined(HAVE_MULTIHARMONIC)
 #define NBR_MAX_HARMONIC    2
+#define MAX_STACK_SIZE0     8
+#define MAX_STACK_SIZE     40
 #else
-#define NBR_MAX_HARMONIC  100
+#define NBR_MAX_HARMONIC   40
+#define MAX_STACK_SIZE0     2
+#define MAX_STACK_SIZE     40
 #endif
 
-#define MAX_STACK_SIZE     40
+// Hereafter, values used for NL circuit + homog (a bit too much for other cases)
+// #define NBR_MAX_HARMONIC  202
+// #define MAX_STACK_SIZE0     2
+// #define MAX_STACK_SIZE    202
+
 
 struct Value {
   int     Type;
@@ -138,8 +150,9 @@ struct MovingBand2D {
 #define SUPPLIST_STARTINGON       2
 #define SUPPLIST_ONONESIDEOF      3
 #define SUPPLIST_ONPOSITIVESIDEOF 4
-#define SUPPLIST_INSUPPORT        5
-#define SUPPLIST_CONNECTEDTO      6
+#define SUPPLIST_ONNEGATIVESIDEOF 5
+#define SUPPLIST_INSUPPORT        6
+#define SUPPLIST_CONNECTEDTO      7
 
 
 /* ------------------------------------------------------------------------ */
@@ -157,6 +170,11 @@ struct Expression {
       int ExpressionIndex_Default;
       int  NumLastRegion;  struct Expression *ExpressionForLastRegion;
     }  PieceWiseFunction;
+    struct {
+      List_T *ExpressionPerRegion;
+      int ExpressionIndex_Default;
+      int  NumLastRegion[2];  struct Expression *ExpressionForLastRegion;
+    }  PieceWiseFunction2;
   }  Case;
 };
 
@@ -164,11 +182,16 @@ struct ExpressionPerRegion {
   int  RegionIndex, ExpressionIndex;
 };
 
+struct ExpressionPerRegion2 {
+  int  RegionIndex[2], ExpressionIndex;
+};
+
 /* Expression.Type */
 #define UNDEFINED_EXP         0
 #define CONSTANT              1
 #define WHOLEQUANTITY         2
 #define PIECEWISEFUNCTION     3
+#define PIECEWISEFUNCTION2    4
 
 
 /* ------------------------------------------------------------------------ */
@@ -183,7 +206,7 @@ struct Constraint {
 };
 
 struct ConstraintPerRegion {
-  int  Type, RegionIndex, SubRegionIndex, TimeFunctionIndex;
+  int  Type, RegionIndex, SubRegionIndex, SubRegion2Index, TimeFunctionIndex;
   union {
     struct { int  ExpressionIndex, ExpressionIndex2; } Fixed;
     struct { char *ResolutionName; } Solve;
@@ -275,6 +298,8 @@ struct JacobianCase {
 #define JACOBIAN_SUR_AXI_SQU_RECT_SHELL  80
 #define JACOBIAN_VOL_AXI_SQU_RECT_SHELL  81
 
+#define JACOBIAN_VOL_UNI_DIR_SHELL       82
+
 #define JACOBIAN_VOL_PLPD_X              90
 
 #define JACOBIAN_VOL_AXI_PLPD_X          100
@@ -450,6 +475,11 @@ struct FunctionActive {
       double *x, *y ;
       double *data ;
     } ListMatrix;
+    struct {
+      int     NbrLinesX, NbrLinesY, NbrLinesZ;
+      double *x, *y, *z ;
+      double *data ;
+    } ListMatrix3D;
   } Case;
 };
 
@@ -588,7 +618,7 @@ struct EquationTerm {
 	int     DefineQuantityIndexNoDof, DofInTrace;
       } Term;
 
-      int  InIndex;
+      int  InIndex, SubRegion;
       int  Full_Matrix;
       int  IntegrationMethodIndex, JacobianMethodIndex;
 
@@ -618,6 +648,8 @@ struct EquationTerm {
 	int     DefineQuantityIndexNoDof;
       } Term;
       int  InIndex;
+      int  SubType;
+      struct FemGlobalTermActive  *Active;
     } GlobalTerm;
 
 
@@ -664,6 +696,12 @@ struct FemLocalTermActive {
   double **Matrix;
 };
 
+struct FemGlobalTermActive {
+  struct QuantityStorage  *QuantityStorageEqu_P;
+  struct QuantityStorage  *QuantityStorageDof_P;
+  int flag_Dof;
+};
+
 struct GlobalQuantityStorage {
   int     NumEquation;
   int     NumDof;
@@ -689,6 +727,11 @@ struct GlobalEquationTerm {
 #define GLOBALEQUATION  3
 #define DERHAM          4
 
+/* EquationTerm.SubType */
+#define EQ_ST_SELF           1
+#define EQ_ST_MUTUAL         2
+#define EQ_ST_SELFANDMUTUAL  3
+
 /* Term.TypeOfTimeDerivative */
 #define NODT_           0
 #define DT_             1
@@ -702,6 +745,13 @@ struct GlobalEquationTerm {
 #define NEVERDT_       11
 #define DTNL_          12
 #define DTDOFJACNL_    13
+#define EIG_           14
+#define NLEIG1DOF_     20
+#define NLEIG2DOF_     21
+#define NLEIG3DOF_     22
+#define NLEIG4DOF_     23
+#define NLEIG5DOF_     24
+#define NLEIG6DOF_     25
 
 /* Term.TypeOperator */
 #define NOOP       0
@@ -733,9 +783,9 @@ struct GlobalEquationTerm {
 #define NSxCURL   17
 #define NSxDIV    18
 
-#define _D1       21
-#define _D2       22
-#define _D3       23
+#define OP_D1     21
+#define OP_D2     22
+#define OP_D3     23
 
 
 /* CanonicalWholeQuantity */
@@ -798,10 +848,8 @@ struct WholeQuantity {
              int InIndex, DofIndexInWholeQuantity; }             Trace;
     struct { char *SystemName; int DefineSystemIndex;
              int DofNumber; }                                    DofValue;
-    struct { List_T *WholeQuantity; // I think this is not used...
-      int Index, FunctionType, NbrArguments, NbrParameters, NbrPoints; } MHTransform;
-    struct { List_T *WholeQuantity; // I think this is not used...
-      int Index, FunctionType, NbrArguments, NbrParameters, NbrPoints, FreqOffSet; } MHJacNL;
+    struct { List_T *WholeQuantity_L; int Index, NbrPoints; }    MHTransform;
+    struct { int Index, FunctionType, NbrArguments, NbrParameters, NbrPoints, FreqOffSet; } MHJacNL;
   } Case;
 
 };
@@ -936,7 +984,7 @@ struct Operation {
     struct {
       char *VariableName;
       int NormType;
-    } GetResidual;
+    } GetNorm;
     struct {
       int CopyFromTimeStep;
     } CreateSolution;
@@ -961,6 +1009,7 @@ struct Operation {
       int     NumEigenvalues;
       double  Shift_r, Shift_i;
       int     FilterExpressionIndex;
+      List_T *RationalCoefsNum, *RationalCoefsDen;
     } EigenSolve;
     struct {
       List_T  *Expressions;
@@ -1091,12 +1140,12 @@ struct Operation {
       int     ExprIndex;
     } SaveMesh;
     struct {
-      char    *Quantity;
+      char    *Quantity, *Quantity2, *Quantity3;
       char    *Name_MshFile;
       int     GeoDataIndex;
       double  Factor;
       int     GroupIndex;
-    } DeformeMesh;
+    } DeformMesh;
     struct {
       char    *String;
     } SetGlobalSolverOptions;
@@ -1104,8 +1153,9 @@ struct Operation {
       List_T  *FieldsToSkip;
     } BroadcastFields;
     struct {
+      int useList;
       char *from, *to;
-    } CopySolution;
+    } Copy;
   } Case;
 
 };
@@ -1160,7 +1210,7 @@ struct IterativeLoopSystem {
 #define OPERATION_CHANGEOFCOORDINATES       8
 #define OPERATION_CHANGEOFCOORDINATES2      9
 #define OPERATION_CREATEDIR                10
-#define OPERATION_DEFORMEMESH              11
+#define OPERATION_DEFORMMESH               11
 #define OPERATION_DELETEFILE               12
 #define OPERATION_DOFSFREQUENCYSPECTRUM    13
 #define OPERATION_EIGENSOLVE               14
@@ -1245,6 +1295,17 @@ struct IterativeLoopSystem {
 #define OPERATION_SETDTIME                 93
 #define OPERATION_REMOVELASTSOLUTION       94
 #define OPERATION_COPYSOLUTION             95
+#define OPERATION_COPYRHS                  96
+#define OPERATION_COPYRESIDUAL             97
+#define OPERATION_COPYINCREMENT            98
+#define OPERATION_COPYDOFS                 99
+#define OPERATION_GETNORMSOLUTION          100
+#define OPERATION_GETNORMRHS               101
+#define OPERATION_GETNORMRESIDUAL          102
+#define OPERATION_GETNORMINCREMENT         103
+#define OPERATION_HPDDMSOLVE               104
+#define OPERATION_BROADCASTVARIABLES       105
+#define OPERATION_DEBUG                    106
 
 /* ChangeOfState.Type */
 #define CHANGEOFSTATE_NOCHANGE              0
@@ -1305,7 +1366,7 @@ struct PostQuantityTerm {
   List_T  *WholeQuantity;
   int     NbrQuantityIndex, *QuantityIndexTable;
   int     *QuantityTraceGroupIndexTable;
-  int     InIndex, JacobianMethodIndex, IntegrationMethodIndex;
+  int     InIndex, SubRegion, JacobianMethodIndex, IntegrationMethodIndex;
 };
 
 /* PostQuantityTerm.Type */
@@ -1351,7 +1412,8 @@ struct PostSubOperation {
   int    StoreInRegister, StoreMinInRegister, StoreMinXinRegister;
   int    StoreMinYinRegister, StoreMinZinRegister, StoreMaxInRegister;
   int    StoreMaxXinRegister, StoreMaxYinRegister, StoreMaxZinRegister;
-  char  *SendToServer, *Color;
+  char  *SendToServer, *Color, *Units;
+  bool   Visible, Closed;
   List_T *SendToServerList;
   int    StoreInField, StoreInMeshBasedField;
   int    Legend, FrozenTimeStepList;
@@ -1444,7 +1506,7 @@ struct PostOpSolutions {
 #define FORMAT_REGION_TABLE           16
 #define FORMAT_REGION_VALUE           17
 #define FORMAT_FREQUENCY_REGION_VALUE 18
-#define FORMAT_UNV                    19
+#define FORMAT_NXUNV                  19
 #define FORMAT_NODE_TABLE             20
 #define FORMAT_LOOP_ERROR             21
 #define FORMAT_GETDP                  22
@@ -1521,10 +1583,11 @@ struct CurrentData {
 
   // For IterativeLoop
   double  Iteration, RelativeDifference, RelativeDifferenceOld;
-  double  RelaxationFactor, Residual;
+  double  RelaxationFactor, Residual, ResidualN, Residual_Iter1; //+++
+  double  RelaxFac, NbrTestedFac; //+++
 
   // Iterative linear system solvers
-  double  KSPIts;
+  double  KSPIterations, KSPIteration, KSPResidual;
 };
 
 /* ------------------------------------------------------------------------ */
@@ -1638,6 +1701,7 @@ struct EntityInTree {
 /* ------------------------------------------------------------------------ */
 
 int fcmp_Integer                  (const void *a, const void *b);
+int fcmp_Integer2                 (const void *a, const void *b);
 int fcmp_Constant                 (const void *a, const void *b);
 int fcmp_Expression_Name          (const void *a, const void *b);
 int fcmp_Group_Name               (const void *a, const void *b);
