@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@onelab.info>.
@@ -63,6 +63,7 @@ struct StringXDefine  FunctionForGroup_SuppList[] = {
   {"StartingOn"      , SUPPLIST_STARTINGON},
   {"OnOneSideOf"     , SUPPLIST_ONONESIDEOF},
   {"OnPositiveSideOf", SUPPLIST_ONPOSITIVESIDEOF},
+  {"OnNegativeSideOf", SUPPLIST_ONNEGATIVESIDEOF},
   {"InSupport"       , SUPPLIST_INSUPPORT},
   {"ConnectedTo"     , SUPPLIST_CONNECTEDTO},
   {NULL              , 0}
@@ -73,6 +74,7 @@ struct StringXDefine1Nbr  Jacobian_Type[] = {
   {"VolSphShell"       , JACOBIAN_VOL_SPH_SHELL         , -1} ,
   {"VolCylShell"       , JACOBIAN_VOL_CYL_SHELL         , -1} ,
   {"VolRectShell"      , JACOBIAN_VOL_RECT_SHELL        , -1} ,
+  {"VolUniDirShell"    , JACOBIAN_VOL_UNI_DIR_SHELL     , -1} ,
   {"VolPlpdX"          , JACOBIAN_VOL_PLPD_X            , 2} ,
 
   {"VolAxi"            , JACOBIAN_VOL_AXI               , 0} ,
@@ -148,6 +150,14 @@ struct StringXDefine  Formulation_Type[] = {
   {NULL             , FEMEQUATION}
  } ;
 
+struct StringXDefine  Equation_SubType[] = {
+  {"Self"          , EQ_ST_SELF},
+  {"Mutual"        , EQ_ST_MUTUAL},
+  {"SelfAndMutual" , EQ_ST_SELFANDMUTUAL},
+  {"MutualAndSelf" , EQ_ST_SELFANDMUTUAL},
+  {NULL            , EQ_ST_SELF}
+ } ;
+
 struct StringXDefine  DefineQuantity_Type[] = {
   /* a supprimer */
   {"LocalQuantity"    , LOCALQUANTITY},
@@ -174,9 +184,9 @@ struct StringXDefine  Operator_Type[] = {
   {"NSxGrad" , NSxGRAD  } ,
   {"NPxGrad" , NPxGRAD  } ,
 
-  {"D1"      , _D1 } ,
-  {"D2"      , _D2 } ,
-  {"D3"      , _D3 } ,
+  {"D1"      , OP_D1 } ,
+  {"D2"      , OP_D2 } ,
+  {"D3"      , OP_D3 } ,
 
   {NULL      , NOOP }
 } ;
@@ -207,7 +217,9 @@ struct StringXDefine  Operation_Type[] = {
   {"GenerateOnly"           , OPERATION_GENERATEONLY},
   {"GenerateOnlyJac"        , OPERATION_GENERATEONLYJAC},
   {"GenerateRHS"            , OPERATION_GENERATERHS},
+  {"GenerateRightHandSide"  , OPERATION_GENERATERHS},
   {"GenerateRHSCumulative"  , OPERATION_GENERATERHS_CUMULATIVE},
+  {"GenerateRightHandSideCumulative"  , OPERATION_GENERATERHS_CUMULATIVE},
   {"GenerateSeparate"       , OPERATION_GENERATESEPARATE},
   {"InitCorrection"         , OPERATION_INITCORRECTION},
   {"InitSolution"           , OPERATION_INITSOLUTION},
@@ -218,8 +230,11 @@ struct StringXDefine  Operation_Type[] = {
   {"SaveSolutions"          , OPERATION_SAVESOLUTIONS},
   {"SetCurrentSystem"       , OPERATION_SETCURRENTSYSTEM},
   {"SetRHSAsSolution"       , OPERATION_SETRHSASSOLUTION},
+  {"SetRightHandSideAsSolution" , OPERATION_SETRHSASSOLUTION},
   {"SetSolutionAsRHS"       , OPERATION_SETSOLUTIONASRHS},
+  {"SetSolutionAsRightHandSide", OPERATION_SETSOLUTIONASRHS},
   {"SwapSolutionAndRHS"     , OPERATION_SWAPSOLUTIONANDRHS},
+  {"SwapSolutionAndRightHandSide" , OPERATION_SWAPSOLUTIONANDRHS},
   {"SwapSolutionAndResidual", OPERATION_SWAPSOLUTIONANDRESIDUAL},
   {"Solve"                  , OPERATION_SOLVE},
   {"SolveAgain"             , OPERATION_SOLVEAGAIN},
@@ -227,8 +242,10 @@ struct StringXDefine  Operation_Type[] = {
   {"SolveJacAgain"          , OPERATION_SOLVEJACAGAIN},
   {"SolveJac_AdaptRelax"    , OPERATION_SOLVEJACADAPTRELAX},
   {"SolveNL"                , OPERATION_SOLVENL},
+  {"HPDDMSolve"             , OPERATION_HPDDMSOLVE},
   {"TransferSolution"       , OPERATION_TRANSFERSOLUTION},
   {"Update"                 , OPERATION_UPDATE},
+  {"Debug"                  , OPERATION_DEBUG},
   {NULL                     , OPERATION_NONE}
 } ;
 
@@ -260,13 +277,18 @@ struct StringXPointer  Current_Value[] = {
 #if defined(HAVE_KERNEL)
   {"Time"     , &Current.Time},       {"DTime"     , &Current.DTime},
   {"Theta"    , &Current.Theta},      {"TimeStep"  , &Current.TimeStep},
-  {"Iteration", &Current.Iteration},  {"Iter"      , &Current.Iteration},
-  {"Residual" , &Current.Residual},
+  {"Iter"      , &Current.Iteration},
+  {"Iteration", &Current.Iteration},  {"NLIteration", &Current.Iteration},
+  {"Residual" , &Current.Residual},   {"NLResidual" , &Current.Residual},
+  {"RelaxFac" , &Current.RelaxFac}, //+++
+  {"ResidualN" , &Current.ResidualN}, //+++
+  {"Residual_Iter1" , &Current.Residual_Iter1}, //+++
+  {"NbrTestedFac" , &Current.NbrTestedFac}, //+++
   {"TimeImag" , &Current.TimeImag},   {"Eigenvalue", &Current.Time},
   {"EigenvalueReal" , &Current.Time}, {"EigenvalueImag" , &Current.TimeImag},
   {"ReOmega"  , &Current.Time},       {"ImOmega"   , &Current.TimeImag},
   {"wr"       , &Current.Time},       {"wi"        , &Current.TimeImag},
-  {"Breakpoint" , &Current.Breakpoint},
+  {"Breakpoint", &Current.Breakpoint},
 
   {"X"  , &Current.x}, {"Y"  , &Current.y},  {"Z"  , &Current.z},
   {"XS" , &Current.xs},{"YS" , &Current.ys}, {"ZS" , &Current.zs},
@@ -284,7 +306,9 @@ struct StringXPointer  Current_Value[] = {
   {"QuadraturePointIndex",  &Current.QuadraturePointIndex},
   {"QP",                    &Current.QuadraturePointIndex},
 
-  {"KSPIts", &Current.KSPIts},
+  {"KSPIterations", &Current.KSPIterations}, {"KSPIts", &Current.KSPIterations},
+  {"KSPIteration", &Current.KSPIteration},
+  {"KSPResidual", &Current.KSPResidual},
 #endif
   {NULL       , NULL}
 } ;
@@ -318,7 +342,8 @@ struct StringXDefine  PostSubOperation_Format[] = {
   {"GetDP"               , FORMAT_GETDP},
   {"Gmsh"                , FORMAT_GMSH},
   {"GmshParsed"          , FORMAT_GMSH_PARSED},
-  {"Unv"                 , FORMAT_UNV},
+  {"Unv"                 , FORMAT_NXUNV},
+  {"NX"                  , FORMAT_NXUNV},
   {"Gnuplot"             , FORMAT_GNUPLOT },
   {"Matlab"              , FORMAT_MATLAB },
   {"Adaptation"          , FORMAT_ADAPT },
@@ -871,7 +896,7 @@ struct StringX3Function3Nbr  BF_Function[] = {
   {"BF_RegionY" , (CAST)BF_RegionY  , (CAST)BF_dRegionY , (CAST)BF_Zero , 0. , ALL, 0 },
   {"BF_RegionZ" , (CAST)BF_RegionZ  , (CAST)BF_dRegionZ , (CAST)BF_Zero , 0. , ALL, 0 },
 
-  {"BF_Global"  , (CAST)BF_Global   , (CAST)BF_dGlobal , (CAST)BF_Zero , 0. , ALL, 0 },
+  {"BF_Global"  , (CAST)BF_Global   , (CAST)BF_dGlobal , (CAST)BF_dInvGlobal , 0. , ALL, 0 },
   {"BF_dGlobal" , (CAST)BF_dGlobal  , (CAST)BF_Zero    , (CAST)BF_Global, 0. , ALL, 0 },
 #endif
 
@@ -909,6 +934,7 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"Cosh"              , (CAST)F_Cosh             ,   0,   1 },
   {"Tanh"              , (CAST)F_Tanh             ,   0,   1 },
   {"Fabs"              , (CAST)F_Fabs             ,   0,   1 },
+  {"Abs"               , (CAST)F_Abs              ,   0,   1 },
   {"Floor"             , (CAST)F_Floor            ,   0,   1 },
   {"Ceil"              , (CAST)F_Ceil             ,   0,   1 },
   {"Atan2"             , (CAST)F_Atan2            ,   0,   2 },
@@ -998,6 +1024,7 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"GetVolume"         , (CAST)F_GetVolume        ,  -1,   0 },
   {"GetNumElement"     , (CAST)F_GetNumElement    ,   0,   0 },
   {"GetNumElements"    , (CAST)F_GetNumElements   ,  -1,   0 },
+  {"GetNumNodes"       , (CAST)F_GetNumNodes      ,   0,   0 },
   {"CellSize"          , (CAST)F_CellSize         ,  -1,   0 },
   {"ProjectPointOnEllipse", (CAST)F_ProjectPointOnEllipse,  2, 1 },
   {"SquNormEdgeValues" , (CAST)F_SquNormEdgeValues,   2,   0 },
@@ -1013,8 +1040,11 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"GetWallClockTime"  , (CAST)F_GetWallClockTime ,   0,   0 },
   {"Memory"            , (CAST)F_GetMemory        ,   0,   0 },
   {"GetMemory"         , (CAST)F_GetMemory        ,   0,   0 },
-  {"SetNumberRunTime"  , (CAST)F_SetNumberRunTime ,   -1,   -1 },
-  {"GetNumberRunTime"  , (CAST)F_GetNumberRunTime ,   -1,   -1 },
+  {"SetNumberRunTime"  , (CAST)F_SetNumberRunTime ,  -1,  -1 },
+  {"GetNumberRunTime"  , (CAST)F_GetNumberRunTime ,  -1,  -1 },
+  {"SetVariable"       , (CAST)F_SetVariable      ,  -1,  -1 },
+  {"SetCumulativeVariable" , (CAST)F_SetCumulativeVariable ,  -1,  -1 },
+  {"GetVariable"       , (CAST)F_GetVariable      ,  -1,  -1 },
   {"VirtualWork"       , (CAST)F_VirtualWork      ,   0,   1 },
   {"Felec"	       , (CAST)F_Felec      	  ,   0,   1 },
 
@@ -1039,16 +1069,20 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"NodeForceDensity"  , (CAST)F_NodeForceDensity ,   0,   1 },
   {"AssDiag"           , (CAST)F_AssDiag          ,  -1,   0 },
 
+  {"AtIndex"           , (CAST)F_AtIndex          ,  -1,   1 },
+
   // F_Interpolation
-  {"InterpolationLinear"   , (CAST)F_InterpolationLinear   ,  -2,   1 },
-  {"dInterpolationLinear"  , (CAST)F_dInterpolationLinear  ,  -2,   1 },
-  {"dInterpolationLinear2" , (CAST)F_dInterpolationLinear2 ,  -2,   1 },
-  {"InterpolationAkima"    , (CAST)F_InterpolationAkima    ,  -2,   1 },
-  {"dInterpolationAkima"   , (CAST)F_dInterpolationAkima   ,  -2,   1 },
-  {"InterpolationBilinear" , (CAST)F_InterpolationBilinear ,  -1,   2 },
-  {"dInterpolationBilinear", (CAST)F_dInterpolationBilinear,  -1,   2 },
-  {"ValueFromIndex"        , (CAST)F_ValueFromIndex        ,  -1,   0 },
-  {"VectorFromIndex"       , (CAST)F_VectorFromIndex       ,  -1,   0 },
+  {"InterpolationLinear"    , (CAST)F_InterpolationLinear    ,  -2,   1 },
+  {"dInterpolationLinear"   , (CAST)F_dInterpolationLinear   ,  -2,   1 },
+  {"dInterpolationLinear2"  , (CAST)F_dInterpolationLinear2  ,  -2,   1 },
+  {"InterpolationAkima"     , (CAST)F_InterpolationAkima     ,  -2,   1 },
+  {"dInterpolationAkima"    , (CAST)F_dInterpolationAkima    ,  -2,   1 },
+  {"InterpolationBilinear"  , (CAST)F_InterpolationBilinear  ,  -1,   2 },
+  {"dInterpolationBilinear" , (CAST)F_dInterpolationBilinear ,  -1,   2 },
+  {"InterpolationTrilinear" , (CAST)F_InterpolationTrilinear ,  -1,   3 },
+  {"dInterpolationTrilinear", (CAST)F_dInterpolationTrilinear,  -1,   3 },
+  {"ValueFromIndex"         , (CAST)F_ValueFromIndex         ,  -1,   0 },
+  {"VectorFromIndex"        , (CAST)F_VectorFromIndex        ,  -1,   0 },
 
   // F_Hysteresis
   {"dhdb_Jiles"            , (CAST)F_dhdb_Jiles            ,  -1,   3 },
@@ -1058,13 +1092,13 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"dhdb_Ducharne"         , (CAST)F_dhdb_Ducharne         ,  -1,   3 },
   {"h_Ducharne"            , (CAST)F_h_Ducharne            ,  -1,   3 },
   {"nu_Ducharne"           , (CAST)F_nu_Ducharne           ,  -1,   3 },
-  {"mu_Vinch"              , (CAST)F_mu_Vinch              ,  -1,   3 },
-  {"nu_Vinch"              , (CAST)F_nu_Vinch              ,  -1,   3 },
-  {"h_Vinch"               , (CAST)F_h_Vinch               ,  -1,   4 },
-  {"dhdb_Vinch"            , (CAST)F_dhdb_Vinch            ,  -1,   3 },
-  {"dbdh_Vinch"            , (CAST)F_dbdh_Vinch            ,  -1,   3 },
-  {"Update_Jk"             , (CAST)F_Update_Jk             ,  -1,   6 },  //kj+++
-  {"Update_Jk_sd"          , (CAST)F_Update_Jk_sd          ,  -1,   6 },  //kj+++
+  //{"mu_Vinch"              , (CAST)F_mu_Vinch              ,  -1,   3 }, // NOT USED FOR NOW (26/06/2016)
+  //{"nu_Vinch"              , (CAST)F_nu_Vinch              ,  -1,   3 }, // NOT USED FOR NOW (26/06/2016)
+  //{"h_Vinch"               , (CAST)F_h_Vinch               ,  -1,   4 }, // NOT USED FOR NOW (26/06/2016)
+  //{"dhdb_Vinch"            , (CAST)F_dhdb_Vinch            ,  -1,   3 }, // NOT USED FOR NOW (26/06/2016)
+  //{"dbdh_Vinch"            , (CAST)F_dbdh_Vinch            ,  -1,   3 }, // NOT USED FOR NOW (26/06/2016)
+  //{"Update_Jk"             , (CAST)F_Update_Jk             ,  -1,   6 },  //kj+++ // NOT USED FOR NOW (26/06/2016)
+  //{"Update_Jk_sd"          , (CAST)F_Update_Jk_sd          ,  -1,   6 },  //kj+++ // NOT USED FOR NOW (26/06/2016)
   {"Update_Cell_K"         , (CAST)F_Update_Cell_K         ,  -1,   4 },  //kj+++
   {"b_Vinch_K"             , (CAST)F_b_Vinch_K             ,  -1,   7 },  // 1+3*2=7 //kj+++
   {"h_Vinch_K"             , (CAST)F_h_Vinch_K             ,  -1,   9 },  // parameter is dimension {2},{3}, 3+3*2=9 //kj+++
@@ -1090,6 +1124,7 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"CurrentPerfectlyConductingSphereMwt",(CAST)F_CurrentPerfectlyConductingSphereMwt,  3, 1 },
 
   {"AcousticFieldSoftSphere",         (CAST)F_AcousticFieldSoftSphere,  2, 1 },
+  {"AcousticFieldSoftSphereABC",      (CAST)F_AcousticFieldSoftSphereABC, 5, 1 },
   {"DrAcousticFieldSoftSphere",       (CAST)F_DrAcousticFieldSoftSphere, 2, 1 },
   {"RCSSoftSphere",                   (CAST)F_RCSSoftSphere,  2, 1 },
   {"AcousticFieldHardSphere",         (CAST)F_AcousticFieldHardSphere,  2, 1 },
@@ -1107,6 +1142,23 @@ struct StringXFunction2Nbr  F_Function[] = {    /* #Par #Arg */
   {"OSRC_Aj",   (CAST)F_OSRC_Aj, 3, 0},
   {"OSRC_Bj",   (CAST)F_OSRC_Bj, 3, 0},
 
+  {"pnm"               , (CAST)F_pnm              ,   0,   3 },
+  {"unm"               , (CAST)F_unm              ,   0,   3 },
+  {"snm"               , (CAST)F_snm              ,   0,   3 },
+  {"Xnm"               , (CAST)F_Xnm              ,   0,   4 },
+  {"Ynm"               , (CAST)F_Ynm              ,   0,   4 },
+  {"Znm"               , (CAST)F_Znm              ,   0,   4 },
+  {"Mnm"               , (CAST)F_Mnm              ,   0,   5 },
+  {"Nnm"               , (CAST)F_Nnm              ,   0,   5 },
+
+  {"DyadGreenHom"      , (CAST)F_DyadGreenHom     ,   0,   6 },
+  {"CurlDyadGreenHom"  , (CAST)F_CurlDyadGreenHom ,   0,   6 },
+
+  // F_PeWe
+  {"ElastodynamicsCylinderCavity", (CAST)F_ElastodynamicsCylinderCavity, 5, 1},
+  {"ElastodynamicsCylinderWall", (CAST)F_ElastodynamicsCylinderWall, 5, 1},
+  {"ElastodynamicsCylinderWallS", (CAST)F_ElastodynamicsCylinderWallS, 5, 1},
+
   // F_Raytracing: ray tracing functions
   {"CylinderPhase",      (CAST)F_CylinderPhase, 0, 1 },
   {"DiamondPhase",       (CAST)F_DiamondPhase,  0, 1 },
