@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@onelab.info>.
@@ -67,17 +67,33 @@ void Cal_InitGalerkinTermOfFemEquation(struct EquationTerm     * EquationTerm_P,
     QuantityStorageNoDof->BasisFunction[0].Dof = DofForNoDof_P ;
   }
 
-  /* Warning: not correct if nonsymmetrical tensor in expression */
+  assDiag_done.clear();
+
+  // check for potentially symmetrical elementary matrix
   FI->SymmetricalMatrix =
     (EquationTerm_P->Case.LocalTerm.Term.DefineQuantityIndexEqu ==
      EquationTerm_P->Case.LocalTerm.Term.DefineQuantityIndexDof) &&
     (EquationTerm_P->Case.LocalTerm.Term.TypeOperatorEqu ==
      EquationTerm_P->Case.LocalTerm.Term.TypeOperatorDof) ;
 
-  assDiag_done.clear();
-
-  if(EquationTerm_P->Case.LocalTerm.Term.CanonicalWholeQuantity_Equ != CWQ_NONE)
-    FI->SymmetricalMatrix = 0 ;
+  if(FI->SymmetricalMatrix){
+    // nonsymmetric if we play with test functions
+    if(EquationTerm_P->Case.LocalTerm.Term.CanonicalWholeQuantity_Equ != CWQ_NONE){
+      FI->SymmetricalMatrix = 0 ;
+    }
+    else{
+      for(int i = 0; i < List_Nbr(EquationTerm_P->Case.LocalTerm.Term.WholeQuantity); i++){
+        struct WholeQuantity *WholeQuantity_P = (struct WholeQuantity*)List_Pointer
+          (EquationTerm_P->Case.LocalTerm.Term.WholeQuantity, i) ;
+        // be on the safe side if we have a (noncommutative) vector product;
+        // FIXME: we should only do this id one of the arguments is a Dof
+        if(WholeQuantity_P->Type == WQ_BINARYOPERATOR &&
+           WholeQuantity_P->Case.Operator.TypeOperator == OP_CROSSPRODUCT)
+          FI->SymmetricalMatrix = 0 ;
+        // FIXME: we should detect nonsymmetric tensors
+      }
+    }
+  }
 
   if (FI->SymmetricalMatrix) {
     FI->Type_FormDof = FI->Type_FormEqu ;
@@ -235,6 +251,14 @@ void Cal_InitGalerkinTermOfFemEquation(struct EquationTerm     * EquationTerm_P,
   case DTDOFJACNL_    : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_DtDofJacNL   ; break;
   case NEVERDT_       : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_NeverDt      ; break;
   case DTNL_          : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_DtNL         ; break;
+  // nleigchange
+  case NLEIG1DOF_      : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_NLEig1Dof   ; break;
+  case NLEIG2DOF_      : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_NLEig2Dof   ; break;
+  case NLEIG3DOF_      : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_NLEig3Dof   ; break;
+  case NLEIG4DOF_      : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_NLEig4Dof   ; break;
+  case NLEIG5DOF_      : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_NLEig5Dof   ; break;
+  case NLEIG6DOF_      : FI->Function_AssembleTerm = (void (*)())Cal_AssembleTerm_NLEig6Dof   ; break;
+
   default             :
     Message::Error("Unknown type of Operator for Galerkin term (%d)",
                    EquationTerm_P->Case.LocalTerm.Term.TypeTimeDerivative);
@@ -773,7 +797,6 @@ void  Cal_GalerkinTermOfFemEquation(struct Element          * Element,
 	    vBFxEqu[i][0] = V2.Val[0];
 	    vBFxEqu[i][1] = V2.Val[1];
 	    vBFxEqu[i][2] = V2.Val[2];
-
 	  }
 
 	} /* for Nbr_Equ */
