@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@onelab.info>.
@@ -100,6 +100,50 @@ void  Get_ValueOfExpression(struct Expression * Expression_P,
        QuantityStorage_P0, u, v, w, Value, NbrArguments, Expression_P->Name) ;
     break ;
 
+  case PIECEWISEFUNCTION2 :
+    //    struct Expression * NextExpression_P;
+    struct ExpressionPerRegion2  * ExpressionPerRegion2_P ;
+    int twoRegions[2];
+
+    if (Current.Region == Expression_P->Case.PieceWiseFunction2.NumLastRegion[0] &&
+        Current.SubRegion == Expression_P->Case.PieceWiseFunction2.NumLastRegion[1]) {
+      NextExpression_P = Expression_P->Case.PieceWiseFunction2.ExpressionForLastRegion;
+    }
+    else {
+      twoRegions[0] = Current.Region;
+      twoRegions[1] = Current.SubRegion;
+      if ((ExpressionPerRegion2_P = (struct ExpressionPerRegion2*)
+	   List_PQuery(Expression_P->Case.PieceWiseFunction2.ExpressionPerRegion,
+		       twoRegions, fcmp_Integer2))) {
+	Expression_P->Case.PieceWiseFunction2.NumLastRegion[0] = Current.Region ;
+	Expression_P->Case.PieceWiseFunction2.NumLastRegion[1] = Current.SubRegion ;
+	NextExpression_P =
+          Expression_P->Case.PieceWiseFunction2.ExpressionForLastRegion =
+	  (struct Expression*)List_Pointer(Problem_S.Expression,
+					   ExpressionPerRegion2_P->ExpressionIndex) ;
+      }
+      else if (Expression_P->Case.PieceWiseFunction.ExpressionIndex_Default >= 0) {
+        NextExpression_P =
+	  (struct Expression*)
+          List_Pointer(Problem_S.Expression,
+                       Expression_P->Case.PieceWiseFunction.ExpressionIndex_Default) ;
+      }
+      else {
+        NextExpression_P = NULL;
+	if(Current.Region == NO_REGION)
+	  Message::Error("Function '%s' undefined in expressions without support",
+                         Expression_P->Name);
+	else
+	  Message::Error("Function '%s' undefined in [ Region %d, SubRegion %d ]",
+                         Expression_P->Name, Current.Region, Current.SubRegion);
+      }
+    }
+
+    Get_ValueOfExpression
+      (NextExpression_P,
+       QuantityStorage_P0, u, v, w, Value, NbrArguments, Expression_P->Name) ;
+    break ;
+
   case UNDEFINED_EXP :
     if(!Flag_WarningUndefined || strcmp(Flag_WarningUndefined, Expression_P->Name)){
       Message::Warning("Undefined expression '%s' (assuming zero)", Expression_P->Name) ;
@@ -274,8 +318,8 @@ void Cal_SolidAngle(int Source, struct Element *Element,
 // There can be at max one "Dof{op qty}" per WholeQuantity, but as
 // many {op qty} as you want.
 
-static std::map<int, Value> ValueSaved ;
-static std::map<std::string, Value> NamedValueSaved ;
+static std::map<int, struct Value> ValueSaved ;
+static std::map<std::string, struct Value> NamedValueSaved ;
 
 void Cal_WholeQuantity(struct Element * Element,
 		       struct QuantityStorage * QuantityStorage_P0,
@@ -302,7 +346,13 @@ void Cal_WholeQuantity(struct Element * Element,
 
   // we could make this dynamic (with std::vector) to reduce stack usage, but
   // the performance hit is important
-  struct Value Stack[8][MAX_STACK_SIZE] ;
+
+  // ==> forcing a reduced size of stack for MH case...
+  // MAX_STACK_SIZE0 = 8 by default, 2 for MH
+  // segmentation violation and out of memory with high number of harmonics
+  // MAX_STACK_SIZE at least MAX_HAR_SIZE if harmonic function in formulation term
+  struct Value Stack[MAX_STACK_SIZE0][MAX_STACK_SIZE] ;
+
 
   WholeQuantity_P0 = (struct WholeQuantity*)List_Pointer(WholeQuantity_L, 0) ;
 
@@ -312,7 +362,7 @@ void Cal_WholeQuantity(struct Element * Element,
   for (i_WQ = 0 ; i_WQ < List_Nbr(WholeQuantity_L) ; i_WQ++) {
 
     if(Index >= MAX_STACK_SIZE){
-      Message::Error("Stack size exceeded (%d)", MAX_STACK_SIZE);
+      Message::Error("Stack size exceeded (%d>%d)", Index, MAX_STACK_SIZE);
       return;
     }
 
@@ -1058,16 +1108,20 @@ void Cal_WholeQuantity(struct Element * Element,
         Multi[Index] = 0 ;
         Index++ ;
       }
-
       Index -= NbrArguments ;
-      Cal_WholeQuantity(Element, QuantityStorage_P0,
-			WholeQuantity_P->Case.MHTransform.WholeQuantity,
-			u, v, w, -1, 0, &Stack[0][Index], NbrArguments, ExpressionName) ;
-
-      MHTransform(Element, QuantityStorage_P0, u, v, w, &Stack[0][Index],
-		  (struct Expression *)List_Pointer(Problem_S.Expression,
-						    WholeQuantity_P->Case.MHTransform.Index),
-		  WholeQuantity_P->Case.MHTransform.NbrPoints) ;
+      {
+        int N = List_Nbr(WholeQuantity_P->Case.MHTransform.WholeQuantity_L);
+        std::vector<struct Value> MH_Values(N);
+        for(int j = 0; j < N; j++){
+          List_T *WQ; List_Read(WholeQuantity_P->Case.MHTransform.WholeQuantity_L, j, &WQ);
+          Cal_WholeQuantity(Element, QuantityStorage_P0, WQ, u, v, w, -1, 0,
+                            &MH_Values[j], NbrArguments, ExpressionName) ;
+        }
+        MHTransform(Element, QuantityStorage_P0, u, v, w, MH_Values,
+                    (struct Expression *)List_Pointer(Problem_S.Expression,
+                                                      WholeQuantity_P->Case.MHTransform.Index),
+                    WholeQuantity_P->Case.MHTransform.NbrPoints, Stack[0][Index]) ;
+      }
       Multi[Index] = 0 ;
       Index++ ;
       break ;
@@ -1141,7 +1195,7 @@ void Cal_WholeQuantity(struct Element * Element,
       else{
         if(TreatmentStatus != _PRE)
           Message::Warning("Empty register %d: assuming zero value",
-                           WholeQuantity_P->Case.ValueSaved.Index);
+                           WholeQuantity_P->Case.ValueSaved.Index + 1);
         Cal_ZeroValue(&Stack[0][Index]);
         Stack[0][Index].Type = SCALAR ;
       }
@@ -1207,5 +1261,21 @@ void Cal_StoreInRegister(struct  Value  *Value, int RegisterIndex)
 void Cal_StoreInVariable(struct  Value  *Value, const char *name)
 {
   Cal_CopyValue(Value, &NamedValueSaved[name]) ;
-  Export_Value(Value, GetDPNumbers[name]);
+  Export_Value(Value, GetDPNumbers[name], 0, false); // don't append
+}
+
+void Cal_GetValueSaved(struct  Value  *Value, const char *name)
+{
+  if(NamedValueSaved.count(name))
+    Cal_CopyValue(&NamedValueSaved[name], Value) ;
+  else{
+    Message::Warning("Unknown current value '$%s': assuming zero value", name);
+    Cal_ZeroValue(Value);
+    Value->Type = SCALAR ;
+  }
+}
+
+std::map<std::string, struct Value> &Get_AllValueSaved()
+{
+  return NamedValueSaved;
 }
