@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@onelab.info>.
@@ -780,6 +780,28 @@ void Cal_ProductValue(struct Value * V1, struct Value * V2, struct Value * R)
     }
     R->Type = VECTOR ;
   }
+  
+  else if (V1->Type == VECTOR && V2->Type == TENSOR) {
+    if (Current.NbrHar == 1) {
+      a1[0] = V1->Val[0]*V2->Val[0] + V1->Val[1]*V2->Val[3] + V1->Val[2]*V2->Val[6];
+      a1[1] = V1->Val[0]*V2->Val[1] + V1->Val[1]*V2->Val[4] + V1->Val[2]*V2->Val[7];
+      a1[2] = V1->Val[0]*V2->Val[2] + V1->Val[1]*V2->Val[5] + V1->Val[2]*V2->Val[8];
+      R->Val[0] = a1[0];
+      R->Val[1] = a1[1];
+      R->Val[2] = a1[2];
+    }
+    else {
+      for (k = 0 ; k < Current.NbrHar ; k += 2) {
+        CMULT(0,0,0); CMULT(0,1,1); CMULT(0,2,2);
+        CMULT(1,3,3); CMULT(1,4,4); CMULT(1,5,5);
+        CMULT(2,6,6); CMULT(2,7,7); CMULT(2,8,8);
+        CPUT3(0,3,6,0);
+        CPUT3(1,4,7,1);
+        CPUT3(2,5,8,2);
+      }
+    }
+    R->Type = VECTOR ;
+  }
 
   else if (V1->Type == TENSOR && V2->Type == TENSOR_DIAG) {
     if (Current.NbrHar == 1) {
@@ -1826,7 +1848,7 @@ void Cal_RotateValue(struct Value *V1, struct Value *V2, struct Value *R)
     }
     break;
 
-#define COMPUTE_A										\
+#define COMPUTE_A                                                                               \
   A.Val[0]= A0*A0*B0+A0*A3*B3+A0*A6*B6+A3*A0*B1+A3*A3*B4+A3*A6*B7+A6*A0*B2+A6*A3*B5+A6*A6*B8;	\
   A.Val[1]= A1*A0*B0+A1*A3*B3+A1*A6*B6+A4*A0*B1+A4*A3*B4+A4*A6*B7+A7*A0*B2+A7*A3*B5+A7*A6*B8;	\
   A.Val[2]= A2*A0*B0+A2*A3*B3+A2*A6*B6+A5*A0*B1+A5*A3*B4+A5*A6*B7+A8*A0*B2+A8*A3*B5+A8*A6*B8;	\
@@ -1885,21 +1907,21 @@ void Cal_RotateValue(struct Value *V1, struct Value *V2, struct Value *R)
 #define B0r  V2->Val[MAX_DIM* k   +0]
 #define B1r  V2->Val[MAX_DIM* k   +1]
 #define B2r  V2->Val[MAX_DIM* k   +2]
-#define B3r  V2->Val[MAX_DIM* k   +3]
-#define B4r  V2->Val[MAX_DIM* k   +4]
-#define B5r  V2->Val[MAX_DIM* k   +5]
-#define B6r  V2->Val[MAX_DIM* k   +6]
-#define B7r  V2->Val[MAX_DIM* k   +7]
-#define B8r  V2->Val[MAX_DIM* k   +8]
+#define B3r  V2->Val[MAX_DIM* k   +1]
+#define B4r  V2->Val[MAX_DIM* k   +3]
+#define B5r  V2->Val[MAX_DIM* k   +4]
+#define B6r  V2->Val[MAX_DIM* k   +2]
+#define B7r  V2->Val[MAX_DIM* k   +4]
+#define B8r  V2->Val[MAX_DIM* k   +5]
 #define B0i  V2->Val[MAX_DIM*(k+1)+0]
 #define B1i  V2->Val[MAX_DIM*(k+1)+1]
 #define B2i  V2->Val[MAX_DIM*(k+1)+2]
-#define B3i  V2->Val[MAX_DIM*(k+1)+3]
-#define B4i  V2->Val[MAX_DIM*(k+1)+4]
-#define B5i  V2->Val[MAX_DIM*(k+1)+5]
-#define B6i  V2->Val[MAX_DIM*(k+1)+6]
-#define B7i  V2->Val[MAX_DIM*(k+1)+7]
-#define B8i  V2->Val[MAX_DIM*(k+1)+8]
+#define B3i  V2->Val[MAX_DIM*(k+1)+1]
+#define B4i  V2->Val[MAX_DIM*(k+1)+3]
+#define B5i  V2->Val[MAX_DIM*(k+1)+4]
+#define B6i  V2->Val[MAX_DIM*(k+1)+2]
+#define B7i  V2->Val[MAX_DIM*(k+1)+4]
+#define B8i  V2->Val[MAX_DIM*(k+1)+5]
 #define AFFECT(i) 										\
   A.Val[MAX_DIM* k   +i] = t0*B0r+t1*B3r+t2*B6r+t3*B1r+t4*B4r+t5*B7r+t6*B2r+t7*B5r+t8*B8r; 	\
   A.Val[MAX_DIM*(k+1)+i] = t0*B0i+t1*B3i+t2*B6i+t3*B1i+t4*B4i+t5*B7i+t6*B2i+t7*B5i+t8*B8i
@@ -2595,7 +2617,8 @@ void Show_Value(struct Value *A)
   }
 }
 
-void Export_Value(struct Value *A, std::vector<double> &out, List_T *harmonics)
+void Export_Value(struct Value *A, std::vector<double> &out, List_T *harmonics,
+                  bool append)
 {
   std::vector<int> har;
   if(harmonics){
@@ -2607,6 +2630,8 @@ void Export_Value(struct Value *A, std::vector<double> &out, List_T *harmonics)
       har.push_back(i);
   }
 
+  if(!append) out.clear();
+
   switch(A->Type){
 
   case SCALAR :
