@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@onelab.info>.
@@ -9,10 +9,11 @@
 #include "DofData.h"
 
 void EigenSolve(struct DofData * DofData_P, int NumEigenvalues,
-		double shift_r, double shift_i, int FilterExpressionIndex);
+                double shift_r, double shift_i, int FilterExpressionIndex,
+                List_T *RationalCoefsNum, List_T *RationalCoefsDen);
 void EigenSolve_ARPACK(struct DofData * DofData_P, int NumEigenvalues,
                        double shift_r, double shift_i, int FilterExpressionIndex);
 void EigenSolve_SLEPC(struct DofData * DofData_P, int NumEigenvalues,
-                      double shift_r, double shift_i, int FilterExpressionIndex);
-
+                      double shift_r, double shift_i, int FilterExpressionIndex,
+                      List_T *RationalCoefsNum, List_T *RationalCoefsDen);
 #endif
