@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@onelab.info>.
@@ -250,6 +250,11 @@ void EigenSolve_ARPACK(struct DofData * DofData_P, int NumEigenvalues,
 #endif
 
   /* Sanity checks */
+  if(DofData_P->Flag_Init[7] || DofData_P->Flag_Init[6] ||
+     DofData_P->Flag_Init[5] || DofData_P->Flag_Init[4]){
+    Message::Error("High order polynomial and non-linear EVP only available with SLEPc");
+    return;
+  }
   if(!DofData_P->Flag_Init[1] || !DofData_P->Flag_Init[3]){
     Message::Error("No System available for EigenSolve: check 'DtDt' and 'GenerateSeparate'");
     return;
