@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@onelab.info>.
@@ -217,9 +217,10 @@ void F_SurfaceArea(F_ARG)
 
   int     Nbr_Element, i_Element ;
   double  Val_Surface ;
-  double  c11, c21, c12, c22, DetJac ;
+  double  c11, c21, c12, c22, c13, c23, DetJac ;
   int     i, k ;
 
+  // FIXME: TODO redo integration when parameters change!
   if (!Fct->Active) {
     Fct->Active = (struct FunctionActive *)Malloc(sizeof(struct FunctionActive)) ;
 
@@ -256,14 +257,18 @@ void F_SurfaceArea(F_ARG)
 	  Get_NodesCoordinatesOfElement(&Element) ;
 	  Get_BFGeoElement(&Element, 0., 0., 0.) ;
 
-	  c11 = c21 = c12 = c22 = 0. ;
+	  c11 = c21 = c12 = c22 = c13 = c23 = 0. ;
 	  for ( i = 0 ; i < Element.GeoElement->NbrNodes ; i++ ) {
 	    c11 += Element.x[i] * Element.dndu[i][0] ;
 	    c21 += Element.x[i] * Element.dndu[i][1] ;
 	    c12 += Element.y[i] * Element.dndu[i][0] ;
 	    c22 += Element.y[i] * Element.dndu[i][1] ;
+	    c13 += Element.z[i] * Element.dndu[i][0] ;
+	    c23 += Element.z[i] * Element.dndu[i][1] ;
 	  }
-	  DetJac = c11 * c22 - c12 * c21 ;
+          DetJac = sqrt( SQU(c11 * c22 - c12 * c21)
+                         + SQU(c13 * c21 - c11 * c23)
+                         + SQU(c12 * c23 - c13 * c22) );
 
 	  if (Element.Type == TRIANGLE || Element.Type == TRIANGLE_2)
 	    Val_Surface += fabs(DetJac) * 0.5 ;
@@ -275,11 +280,13 @@ void F_SurfaceArea(F_ARG)
 	  Get_NodesCoordinatesOfElement(&Element) ;
 	  Get_BFGeoElement(&Element, 0., 0., 0.) ;
 
-	  c11 = 0. ;
+	  c11 = c12 = c13 = 0. ;
 	  for ( i = 0 ; i < Element.GeoElement->NbrNodes ; i++ ) {
 	    c11 += Element.x[i] * Element.dndu[i][0] ;
+	    c12 += Element.y[i] * Element.dndu[i][0] ;
+	    c13 += Element.z[i] * Element.dndu[i][0] ;
 	  }
-	  DetJac = c11 ;
+          DetJac = sqrt(SQU(c11)+SQU(c12)+SQU(c13));
 
           Val_Surface += fabs(DetJac) * 2 ; // SurfaceArea of LINE x 1m
         }
@@ -413,20 +420,37 @@ void F_GetNumElements(F_ARG)
 
   if (!Fct->Active) {
     Fct->Active = (struct FunctionActive *)Malloc(sizeof(struct FunctionActive)) ;
-    List_T  * InitialList_L = 0;
+    List_T  * InitialList_L;
     if (Fct->NbrParameters == 1) {
       int Index_Region = (int)(Fct->Para[0]) ;
       InitialList_L = List_Create(1, 1, sizeof(int));
       List_Add(InitialList_L, &Index_Region);
     }
+    else if (Fct->NbrParameters > 1) {
+      InitialList_L = List_Create(Fct->NbrParameters,1,sizeof(int));
+      List_Reset(InitialList_L);
+      for (int i=0; i<Fct->NbrParameters; i++) {
+        int Index_Region = (int)(Fct->Para[i]) ;
+        List_Add(InitialList_L, &Index_Region);
+      }
+    }
+    else {
+      InitialList_L = NULL ;
+    }
+
     int Count = 0. ;
     int Nbr_Element = Geo_GetNbrGeoElements() ;
-    for (int i_Element = 0 ; i_Element < Nbr_Element; i_Element++) {
-      Element.GeoElement = Geo_GetGeoElement(i_Element) ;
-      if ((InitialList_L &&
-	   List_Search(InitialList_L, &(Element.GeoElement->Region), fcmp_int)) ||
-	  (!InitialList_L && Element.GeoElement->Region == Current.Region)) {
-	Count++;
+    if(!InitialList_L){
+      Count = Nbr_Element ;
+    }
+    else{
+      for (int i_Element = 0 ; i_Element < Nbr_Element; i_Element++) {
+        Element.GeoElement = Geo_GetGeoElement(i_Element) ;
+        if ((InitialList_L &&
+             List_Search(InitialList_L, &(Element.GeoElement->Region), fcmp_int)) ||
+            (!InitialList_L && Element.GeoElement->Region == Current.Region)) {
+          Count++;
+        }
       }
     }
     Fct->Active->Case.GetNumElements.Value = Count ;
@@ -442,6 +466,18 @@ void F_GetNumElements(F_ARG)
   }
 }
 
+void F_GetNumNodes(F_ARG)
+{
+  // TODO: accept arguments to limit to some regions
+  V->Type = SCALAR ;
+  V->Val[0] = Geo_GetNbrGeoNodes() ;
+  V->Val[MAX_DIM] = 0.;
+  for (int k = 2 ; k < std::min(NBR_MAX_HARMONIC, Current.NbrHar) ; k += 2) {
+    V->Val[MAX_DIM* k] = V->Val[0] ;
+    V->Val[MAX_DIM* (k+1)] = 0. ;
+  }
+}
+
 void F_CellSize(F_ARG)
 {
   double cellSize, Vol;
