@@ -1,9 +1,10 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@onelab.info>.
 
 #include <stdlib.h>
+#include <string.h>
 #include <math.h>
 #include "GetDPConfig.h"
 #include "ProData.h"
@@ -11,6 +12,7 @@
 #include "F.h"
 #include "Message.h"
 #include "Cal_Value.h"
+#include "Cal_Quantity.h"
 #include "OS.h"
 
 extern struct CurrentData Current ;
@@ -127,6 +129,86 @@ void F_GetNumberRunTime (F_ARG)
     V->Val[0] = Message::GetOnelabNumber(Fct->String);
 }
 
+void F_SetVariable (F_ARG)
+{
+  if(!Fct->String){
+    Message::Error("Missing runtime variable name: use SetVariable[...]{$name}");
+    return;
+  }
+
+  if(Fct->NbrArguments < 1){
+    Message::Error("Missing argument in SetVariable[...]{$name}");
+    return;
+  }
+
+  Cal_CopyValue(A, V);
+
+  char tmp[256];
+  strcpy(tmp, Fct->String);
+  for(int i = 1; i < Fct->NbrArguments; i++){
+    if((A+i)->Type != SCALAR){
+      Message::Error("Non-scalar argument in SetVariable");
+      return;
+    }
+    char tmp2[256];
+    sprintf(tmp2, "_%g", (A+i)->Val[0]);
+    strcat(tmp, tmp2);
+  }
+  Cal_StoreInVariable(V, tmp);
+}
+
+void F_SetCumulativeVariable (F_ARG)
+{
+  if(Fct->NbrArguments < 1){
+    Message::Error("Missing argument in SetCumulativeVariable[...]{$name}");
+    return;
+  }
+
+  Cal_CopyValue(A, V);
+
+  char tmp[256];
+  strcpy(tmp, Fct->String);
+  for(int i = 1; i < Fct->NbrArguments; i++){
+    if((A+i)->Type != SCALAR){
+      Message::Error("Non-scalar argument in SetCumulativeVariable");
+      return;
+    }
+    char tmp2[256];
+    sprintf(tmp2, "_%g", (A+i)->Val[0]);
+    strcat(tmp, tmp2);
+  }
+
+  struct Value V_saved ;
+  Cal_GetValueSaved(&V_saved, tmp);
+  Cal_AddValue(V, &V_saved, V);
+
+  Cal_StoreInVariable(V, tmp);
+}
+
+
+
+void F_GetVariable (F_ARG)
+{
+  if(!Fct->String){
+    Message::Error("Missing runtime variable name: use GetVariable[...]{$name}");
+    return;
+  }
+
+  char tmp[256];
+  strcpy(tmp, Fct->String);
+  for(int i = 0; i < Fct->NbrArguments; i++){
+    if((A+i)->Type != SCALAR){
+      Message::Error("Non-scalar argument in GetVariable");
+      return;
+    }
+    char tmp2[256];
+    sprintf(tmp2, "_%g", (A+i)->Val[0]);
+    strcat(tmp, tmp2);
+  }
+
+  Cal_GetValueSaved(V, tmp);
+}
+
 void F_VirtualWork (F_ARG)
 {
   MATRIX3x3 Jac ;
@@ -1364,3 +1446,30 @@ void F_AssDiag(F_ARG)
 
   V->Type = SCALAR ;
 }
+
+void F_AtIndex(F_ARG)
+{
+  int index = 0;
+  double ret = 0.;
+  if(A->Type != SCALAR){
+    Message::Error("Non scalar argument for function 'AtIndex");
+  }
+  else{
+    index = (int)A->Val[0];
+    if (index < 0 || index >= Fct->NbrParameters){
+      Message::Error("Wrong index in function 'AtIndex': %d not in [0,%d[",
+                     index, Fct->NbrParameters);
+    }
+    else{
+      ret = Fct->Para[index];
+    }
+  }
+
+  V->Type = SCALAR;
+  V->Val[0] = ret;
+  if (Current.NbrHar != 1){
+    V->Val[MAX_DIM] = 0. ;
+    for (int k = 2 ; k < std::min(NBR_MAX_HARMONIC, Current.NbrHar) ; k += 2)
+      V->Val[MAX_DIM*k] = V->Val[MAX_DIM*(k+1)] = 0. ;
+  }
+}
