@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@onelab.info>.
@@ -21,6 +21,8 @@ extern List_T  *PreResolutionIndex_L ;
 
 extern int  Nbr_ElementaryBF ;
 
+static double TOL ;
+
 /* ------------------------------------------------------------------------ */
 /*  T r e a t m e n t _ C o n s t r a i n t F o r E l e m e n t             */
 /* ------------------------------------------------------------------------ */
@@ -105,8 +107,7 @@ void  Treatment_ConstraintForElement(struct FunctionSpace    * FunctionSpace_P,
 	      Get_LinkForConstraint
 		(Constraint_P,
 		 abs(Num_Entity[i_Entity]),
-		 &QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].
-		 CodeEntity_Link,
+		 &QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].CodeEntity_Link,
                  QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].BasisFunction->Orient,
 		 QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].Value) ;
 	      if (abs(Num_Entity[i_Entity]) ==
@@ -568,7 +569,7 @@ void  Generate_LinkNodes(struct ConstraintInFS * Constraint_P,
 			 List_T * Couples_L)
 {
   int  Nbr_Entity, i, Nbr_EntityRef, Flag_Filter ;
-  double TOL ;
+  //  double TOL ;
   struct TwoIntOneDouble  TwoIntOneDouble ;
   struct NodeXYZ  NodeXYZ, NodeXYZRef ;
   List_T  * NodeXYZ_L, * NodeXYZRef_L ;
@@ -690,7 +691,7 @@ void  Generate_LinkNodes(struct ConstraintInFS * Constraint_P,
 
 int fcmp_XYZ(const void * a, const void * b)
 {
-  double Result, TOL=Current.GeoData->CharacteristicLength * 1.e-8 ;
+  double Result; //, TOL=Current.GeoData->CharacteristicLength * 1.e-8 ;
 
   if (fabs(Result = ((struct NodeXYZ *)a)->x - ((struct NodeXYZ *)b)->x) > TOL)
     return (Result > 0.)? 1 : -1 ;
@@ -1051,10 +1052,6 @@ void  Generate_LinkFacets(struct ConstraintInFS * Constraint_P,
     return;
   }
 
-  Message::Debug("(ajout) Image: f%d, n%d - n%d - n%d", FacetNNN.NumFacet,
-                 FacetNNN.Node1, FacetNNN.Node2, FacetNNN.Node3) ;
-
-
   for (i = 0 ; i < Nbr_Entity ; i++) {
     List_Read(ExtendedList_L, i, &FacetNNN) ;
     if (!(ExtendedSuppListRef_L &&
