@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@onelab.info>.
@@ -113,11 +113,11 @@ void LinAlg_InitializeSolver(int* argc, char*** argv)
 
 void LinAlg_FinalizeSolver()
 {
-  // this causes random crashes with PETSC <= 3.6 when doing several
+  // this causes random crashes with PETSC 3 when doing several
   // initialize/finalize calls (when using getdp as a library). Until we figure
   // out what's happening, let's simply initialize petsc/slepc once, and never
   // finalize.
-#if (PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR <= 6)
+#if (PETSC_VERSION_MAJOR == 3)  && (PETSC_VERSION_MINOR <= 7)
   return;
 #endif
 
@@ -200,14 +200,15 @@ static void _fillseq(gVector *V)
 
 void LinAlg_CreateMatrix(gMatrix *M, gSolver *Solver, int n, int m)
 {
-  PetscInt prealloc = 100;
-  PetscInt prealloc_full = n;
-  int nonloc = Current.DofData->NonLocalEquations.size();
 
-  // heuristic for preallocation of global rows: don't prelloc more than 100 Mb
-  int onegig = (int)(100 * 1024 * 1024 / (gSCALAR_SIZE * sizeof(double)));
-  if(nonloc * n > onegig){
-    prealloc_full = (int)(onegig / nonloc);
+  PetscInt prealloc = 100. ;
+  PetscInt prealloc_full = n ;
+  int nonloc = Current.DofData->NonLocalEquations.size();
+  // heuristic for preallocation of global rows: don't prelloc more than 500 Mb
+  double limit = 500. * 1024. * 1024. / (gSCALAR_SIZE * sizeof(double));
+  double estim = (double)nonloc * (double)n;
+  if(estim > limit){
+    prealloc_full = (int)(limit / nonloc);
     Message::Debug("Heuristic -petsc_prealloc_full changed to %d", prealloc_full);
   }
 
@@ -220,7 +221,6 @@ void LinAlg_CreateMatrix(gMatrix *M, gSolver *Solver, int n, int m)
   prealloc_full = (n < prealloc_full) ? n : prealloc_full;
 
   std::vector<PetscInt> nnz(n, prealloc);
-  //std::vector<PetscInt> nnz(n, n); // kj+++
 
   // preallocate non local equations as full lines (this is not
   // optimal, but preallocating too few elements leads to horrible
@@ -343,10 +343,12 @@ void LinAlg_ScanScalar(FILE *file, gScalar *S)
 {
 #if defined(PETSC_USE_COMPLEX)
   double a, b;
-  fscanf(file, "%lf %lf", &a, &b);
+  if(fscanf(file, "%lf %lf", &a, &b) != 2)
+    Message::Error("Could not scan scalar");
   S->s = a + PETSC_i * b;
 #else
-  fscanf(file, "%lf", &S->s);
+  if(fscanf(file, "%lf", &S->s) != 1)
+    Message::Error("Could not scan scalar");
 #endif
 }
 
@@ -358,11 +360,13 @@ void LinAlg_ScanVector(FILE *file, gVector *V)
     PetscScalar tmp;
 #if defined(PETSC_USE_COMPLEX)
     double a, b;
-    fscanf(file, "%lf %lf", &a, &b);
+    if(fscanf(file, "%lf %lf", &a, &b) != 2)
+      Message::Error("Could not read data in vector");
     tmp = a + PETSC_i * b;
 #else
     double a;
-    fscanf(file, "%lf", &a);
+    if(fscanf(file, "%lf", &a) != 1)
+      Message::Error("Could not read data in vector");
     tmp = a;
 #endif
     _try(VecSetValues(V->V, 1, &i, &tmp, INSERT_VALUES));
@@ -386,8 +390,9 @@ void LinAlg_ReadVector(FILE *file, gVector *V)
   _try(VecGetSize(V->V, &n));
   PetscScalar *tmp = (PetscScalar*)Malloc(n*sizeof(PetscScalar));
   fread(tmp, sizeof(PetscScalar), n, file);
-  for(PetscInt i = 0; i < n; i++)
+  for(PetscInt i = 0; i < n; i++){
     _try(VecSetValues(V->V, 1, &i, &tmp[i], INSERT_VALUES));
+  }
   LinAlg_AssembleVector(V);
   Free(tmp);
 }
@@ -1215,7 +1220,7 @@ static void _zitsol(gMatrix *A, gVector *B, gVector *X)
   else
     Message::Info("Converged in %d iterations", its);
 
-  Current.KSPIts = its;
+  Current.KSPIterations = its;
 
   for(PetscInt i = 0; i < n; i++){
     PetscScalar d;
@@ -1238,7 +1243,8 @@ static void _zitsol(gMatrix *A, gVector *B, gVector *X)
 static PetscErrorCode _myKspMonitor(KSP ksp, PetscInt it, PetscReal rnorm, void *mctx)
 {
   Message::Info("%3ld KSP Residual norm %14.12e", (long)it, rnorm);
-  // Current.KSPResidual = rnorm;
+  Current.KSPIteration = it;
+  Current.KSPResidual = rnorm;
   return 0;
 }
 
@@ -1329,6 +1335,7 @@ static void _solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
   }
   else if(precond){
 #if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 5)))
+    _try(KSPSetReusePreconditioner(Solver->ksp[kspIndex], PETSC_FALSE));
     _try(KSPSetOperators(Solver->ksp[kspIndex], A->M, A->M));
 #else
     _try(KSPSetOperators(Solver->ksp[kspIndex], A->M, A->M, DIFFERENT_NONZERO_PATTERN));
@@ -1353,7 +1360,7 @@ static void _solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
   if(!Message::GetCommRank() || !Message::GetIsCommWorld()){
     if(its > 1) Message::Info("%d iterations", its);
   }
-  Current.KSPIts = its;
+  Current.KSPIterations = its;
 }
 
 void LinAlg_Solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
