@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@onelab.info>.
@@ -43,7 +43,7 @@ static void Info(int level, char *arg0)
   case 0 :
     fprintf(stderr,
 	    "GetDP, a General environment for the treatment of Discrete Problems\n"
-	    "Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege\n"
+	    "Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege\n"
 	    "Usage: %s [file] [options]\n"
 	    "Processing options:\n"
 	    "  -pre 'Resolution'         pre-processing\n"
@@ -81,8 +81,8 @@ static void Info(int level, char *arg0)
      	    "  -cpu                      report CPU times for all operations\n"
 	    "  -p num                    set progress indicator update (default: 10)\n"
 	    "  -onelab name [address]    communicate with ONELAB (file or server address)\n"
-            "  -setnumber name value     set constant number name=value\n"
-            "  -setstring name value     set constant string name=value\n"
+            "  -setnumber name value     set constant number name=value (or -sn)\n"
+            "  -setstring name value     set constant string name=value (or -ss)\n"
 	    "  -version                  show version number\n"
 	    "  -info                     show detailed version information\n"
 	    "  -help                     show this message\n", arg0);
@@ -169,7 +169,8 @@ static void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *
         }
       }
 
-      else if (!strcmp(argv[i]+1, "setnumber")) {
+      else if (!strcmp(argv[i]+1, "setnumber") ||
+               !strcmp(argv[i]+1, "sn")) {
         i++;
 	if (i + 1 < argc && argv[i][0] != '-') {
           CommandLineNumbers[argv[i]] = std::vector<double>(1, atof(argv[i + 1]));
@@ -180,7 +181,8 @@ static void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *
         }
       }
 
-      else if (!strcmp(argv[i]+1, "setstring")) {
+      else if (!strcmp(argv[i]+1, "setstring") ||
+               !strcmp(argv[i]+1, "ss")) {
         i++;
 	if (i + 1 < argc && argv[i][0] != '-' && argv[i + 1][0] != '-') {
           CommandLineStrings[argv[i]] = std::vector<std::string>(1, argv[i + 1]);
@@ -231,6 +233,14 @@ static void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *
         Message::SetInfoCpu(true); i++;
       }
 
+      else if(!strcmp(argv[i] + 1, "nt")) {
+        i++;
+        if(argv[i])
+          Message::SetNumThreads(atoi(argv[i++]));
+        else
+          Message::Error("Missing number");
+      }
+
       else if (!strcmp(argv[i]+1, "help")  || !strcmp(argv[i]+1, "h") ||
 	       !strcmp(argv[i]+1, "-help") || !strcmp(argv[i]+1, "-h")) {
 	Info(0, argv[0]);
@@ -509,139 +519,30 @@ static void Free_GlobalVariables()
   Free_ParserVariables();
 }
 
-int MainKernel(int argc, char *argv[])
+void GetDPPrintNumbers()
 {
-  if(argc < 2) Info(0, argv[0]);
-
-  std::string cmdline("");
-  for(int i = 0; i < argc; i++){
-    if(i) cmdline += " ";
-    cmdline += argv[i];
-  }
-
-  Message::Initialize(argc, argv);
-
-  char pro[256];
-  char **sargv = (char**)Malloc(256 * sizeof(char*));
-  int sargc, lres = 0, lpos = 0, check = 0;
-  Get_Options(argc, argv, &sargc, sargv, pro, &lres, &lpos, &check);
-
-  if(Message::GetErrorCount()){
-    Message::Finalize();
-    return Message::GetErrorCount();
-  }
-  Message::Info("Running '%s' [GetDP %s, %d node%s]", cmdline.c_str(),
-                GETDP_VERSION, Message::GetCommSize(),
-                Message::GetCommSize() > 1 ? "s" : "");
-  Message::Cpu(3, true, true, true, true, "Started");
-
-  if(sargc > 1){
-    std::string solveropt("");
-    for(int i = 1; i < sargc; i++){
-      if(i > 1) solveropt += " ";
-      solveropt += sargv[i];
+  for(std::map<std::string, std::vector<double> >::iterator
+        it = GetDPNumbers.begin(); it != GetDPNumbers.end(); it++){
+    printf("%s() = {", it->first.c_str());
+    for(unsigned int i = 0; i < it->second.size(); i++){
+      if(i) printf(", ");
+      printf("%g", it->second[i]);
     }
-    Message::Debug("Passing unused options to solver: '%s'", solveropt.c_str());
+    printf("};\n");
   }
-
-  if(!Name_ResFile[0]){
-    Name_ResFile[0] = (char*)Malloc((strlen(Name_Generic)+5)*sizeof(char));
-    strcpy(Name_ResFile[0], Name_Generic);
-    strcat(Name_ResFile[0], ".res");
-    Name_ResFile[1] = 0;
-  }
-
-  if(!Name_MshFile){
-    std::string name = Message::GetOnelabString("Gmsh/MshFileName");
-    if(name.size()){
-      Name_MshFile = strSave(name.c_str());
-      Message::Info("Got mesh name from Onelab: '%s'", Name_MshFile);
-    }
-  }
-  if(!Name_MshFile){
-    Name_MshFile = (char*)Malloc((strlen(Name_Generic)+5)*sizeof(char));
-    strcpy(Name_MshFile, Name_Generic);
-    strcat(Name_MshFile, ".msh");
-  }
-
-#if defined(HAVE_GMSH)
-  Message::Info("Initializing Gmsh");
-  GmshInitialize();
-  if(!GmshGetMessageHandler() && !Flag_CALLED_WITH_ONELAB_SERVER){
-    // do not set msg handler if one is provided (e.g. on Android/iOS)
-    GmshMsg c;
-    GmshSetMessageHandler(&c);
-  }
-  int j = 0;
-  while(Name_GmshReadFile[j]){
-    if(Tag_GmshReadFile[j] >= 0){
-      PView::setGlobalTag(Tag_GmshReadFile[j]);
-      Message::Info("GmshRead[%s] -> View[%d]", Name_GmshReadFile[j], Tag_GmshReadFile[j]);
-    }
-    else{
-      Message::Info("GmshRead[%s]", Name_GmshReadFile[j]);
-    }
-    GmshMergePostProcessingFile(Name_GmshReadFile[j]);
-    j++;
-  }
-#endif
-
-  IncreaseStackSize();
-  LinAlg_InitializeSolver(&sargc, &sargv);
-
-  Init_ProblemStructure();
-  Read_ProblemStructure(pro);
-  Finalize_ProblemStructure();
-
-  int choose = 1;
-  if (!Flag_PRE && !Flag_CAL && !Flag_POS && !check){
-    lres = lpos = 1;
-    choose = 0;
-  }
-
-  if(lres)
-    Print_ListResolution(choose, lres, &Name_Resolution);
-
-  if(lpos)
-    Print_ListPostOperation(choose, lpos, Name_PostOperation);
-
-  if(check){
-    Print_ProblemStructure();
-  }
-  else{
-    check = Message::GetOnelabNumber("GetDP/}ModelCheck");
-    if(check) Print_Object(check - 1);
-  }
-
-  if(Flag_PRE || Flag_CAL || Flag_POS)
-    SolvingAnalyse();
-
-  LinAlg_FinalizeSolver();
-
-  Message::PrintErrorCounter("Run");
-  Message::Cpu(3, true, true, true, true, "Stopped");
-
-#if defined(HAVE_GMSH)
-  if(!Flag_CALLED_WITH_ONELAB_SERVER) GmshFinalize();
-#endif
-
-  Free_GlobalVariables();
-  Free(sargv);
-  Message::Finalize();
-  return Message::GetErrorCount();
 }
 
-int GetDP(const std::vector<std::string> &args, void *ptr)
+void GetDPPrintStrings()
 {
-  onelab::server *onelabServer = (onelab::server*) ptr;
-  if(onelabServer != NULL){
-    onelab::server::setInstance(onelabServer);
-    Flag_CALLED_WITH_ONELAB_SERVER = 1;
+  for(std::map<std::string, std::vector<std::string> >::iterator
+        it = GetDPStrings.begin(); it != GetDPStrings.end(); it++){
+    printf("%s() = {", it->first.c_str());
+    for(unsigned int i = 0; i < it->second.size(); i++){
+      if(i) printf(", ");
+      printf("%s", it->second[i].c_str());
+    }
+    printf("};\n");
   }
-  int argc = args.size();
-  std::vector<char*> argv(argc + 1, (char*)0);
-  for(int i = 0; i < argc; i++) argv[i] = (char*)args[i].c_str();
-  return MainKernel(argc, &argv[0]);
 }
 
 void GetDPClearNumbers()
@@ -687,3 +588,147 @@ std::vector<std::string> &GetDPGetString(const std::string &name)
 {
   return GetDPStrings[name];
 }
+
+int MainKernel(int argc, char *argv[])
+{
+  if(argc < 2) Info(0, argv[0]);
+
+  std::string cmdline("");
+  for(int i = 0; i < argc; i++){
+    if(i) cmdline += " ";
+    cmdline += argv[i];
+  }
+
+  Message::Initialize(argc, argv);
+
+  char pro[256];
+  char **sargv = (char**)Malloc(256 * sizeof(char*));
+  int sargc, lres = 0, lpos = 0, check = 0;
+  Get_Options(argc, argv, &sargc, sargv, pro, &lres, &lpos, &check);
+
+  if(Message::GetErrorCount()){
+    Message::Finalize();
+    return Message::GetErrorCount();
+  }
+  Message::Info("Running '%s' [GetDP %s, %d node%s, max. %d thread%s]",
+                cmdline.c_str(), GETDP_VERSION,
+                Message::GetCommSize(), Message::GetCommSize() > 1 ? "s" : "",
+                Message::GetMaxThreads(), Message::GetMaxThreads() > 1 ? "s" : "");
+  Message::Cpu(3, true, true, true, true, true, "Started");
+
+  if(sargc > 1){
+    std::string solveropt("");
+    for(int i = 1; i < sargc; i++){
+      if(i > 1) solveropt += " ";
+      solveropt += sargv[i];
+    }
+    Message::Debug("Passing unused options to solver: '%s'", solveropt.c_str());
+  }
+
+  if(!Name_ResFile[0]){
+    Name_ResFile[0] = (char*)Malloc((strlen(Name_Generic)+5)*sizeof(char));
+    strcpy(Name_ResFile[0], Name_Generic);
+    strcat(Name_ResFile[0], ".res");
+    Name_ResFile[1] = 0;
+  }
+
+  if(!Name_MshFile){
+    std::string name = Message::GetOnelabString("Gmsh/MshFileName");
+    if(name.size()){
+      Name_MshFile = strSave(name.c_str());
+      Message::Info("Got mesh name from Onelab: '%s'", Name_MshFile);
+    }
+  }
+  if(!Name_MshFile){
+    Name_MshFile = (char*)Malloc((strlen(Name_Generic)+5)*sizeof(char));
+    strcpy(Name_MshFile, Name_Generic);
+    strcat(Name_MshFile, ".msh");
+  }
+
+#if defined(HAVE_GMSH)
+  Message::Info("Initializing Gmsh");
+  GmshInitialize();
+  GmshMsg *msg = 0;
+  if(!GmshGetMessageHandler() && !Flag_CALLED_WITH_ONELAB_SERVER){
+    // do not set msg handler if one is provided (e.g. on Android/iOS)
+    msg = new GmshMsg;
+    GmshSetMessageHandler(msg);
+  }
+  int j = 0;
+  while(Name_GmshReadFile[j]){
+    if(Tag_GmshReadFile[j] >= 0){
+      PView::setGlobalTag(Tag_GmshReadFile[j]);
+      Message::Info("GmshRead[%s] -> View[%d]", Name_GmshReadFile[j], Tag_GmshReadFile[j]);
+    }
+    else{
+      Message::Info("GmshRead[%s]", Name_GmshReadFile[j]);
+    }
+    GmshMergePostProcessingFile(Name_GmshReadFile[j]);
+    j++;
+  }
+#endif
+
+  IncreaseStackSize();
+  LinAlg_InitializeSolver(&sargc, &sargv);
+
+  Init_ProblemStructure();
+  Read_ProblemStructure(pro);
+  Finalize_ProblemStructure();
+
+  int choose = 1;
+  if (!Flag_PRE && !Flag_CAL && !Flag_POS && !check){
+    lres = lpos = 1;
+    choose = 0;
+  }
+
+  if(lres)
+    Print_ListResolution(choose, lres, &Name_Resolution);
+
+  if(lpos)
+    Print_ListPostOperation(choose, lpos, Name_PostOperation);
+
+  if(check){
+    Print_ProblemStructure();
+  }
+  else{
+    check = Message::GetOnelabNumber
+      (Message::GetOnelabClientName() + "/}ModelCheck");
+    if(check) Print_Object(check - 1);
+  }
+
+  if(Flag_PRE || Flag_CAL || Flag_POS)
+    SolvingAnalyse();
+
+  LinAlg_FinalizeSolver();
+
+  Message::PrintErrorCounter("Run");
+  Message::Cpu(3, true, true, true, true, true, "Stopped");
+
+  if(Message::GetVerbosity() == 99){ // debug
+    GetDPPrintNumbers();
+    GetDPPrintStrings();
+  }
+
+#if defined(HAVE_GMSH)
+  if(!Flag_CALLED_WITH_ONELAB_SERVER) GmshFinalize();
+  if(msg) delete msg;
+#endif
+
+  Free_GlobalVariables();
+  Free(sargv);
+  Message::Finalize();
+  return Message::GetErrorCount();
+}
+
+int GetDP(const std::vector<std::string> &args, void *ptr)
+{
+  onelab::server *onelabServer = (onelab::server*) ptr;
+  if(onelabServer != NULL){
+    onelab::server::setInstance(onelabServer);
+    Flag_CALLED_WITH_ONELAB_SERVER = 1;
+  }
+  int argc = args.size();
+  std::vector<char*> argv(argc + 1, (char*)0);
+  for(int i = 0; i < argc; i++) argv[i] = (char*)args[i].c_str();
+  return MainKernel(argc, &argv[0]);
+}
