@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@onelab.info>.
@@ -414,10 +414,12 @@ void  Pos_Formulation(struct Formulation       *Formulation_P,
       }
     }
     else {
-      if((PostStream = FOpen(PostFileName, Flag_BIN ? "ab" : "a")))
-	Message::Direct(4, "          >> '%s'", PostFileName) ;
+      if((PostStream = FOpen(PostFileName, Flag_BIN ?
+                             (PostSubOperation_P->Format == FORMAT_NXUNV ? "r+b" : "ab") :
+                             "a")))
+        Message::Direct(4, "          >> '%s'", PostFileName) ;
       else{
-	Message::Error("Unable to open file '%s'", PostFileName) ;
+        Message::Error("Unable to open file '%s'", PostFileName) ;
         PostStream = stdout ;
       }
     }
@@ -514,8 +516,8 @@ void  Pos_Formulation(struct Formulation       *Formulation_P,
         Message::SendMergeFileRequest(PostFileName);
       }
       // Add link to file
-      Message::AddOnelabStringChoice(Message::GetOnelabClientName() + "/9Output files",
-                                     "file", PostFileName);
+      Message::AddOnelabStringChoice(Message::GetOnelabClientName() + "/{Output files",
+                                     "file", PostFileName, true, true);
     }
 
     /* NewCoordinates print option: write a new mesh */
