@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2016 P. Dular and C. Geuzaine, University of Liege
+// GetDP - Copyright (C) 1997-2017 P. Dular and C. Geuzaine, University of Liege
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <getdp@onelab.info>.
@@ -1462,7 +1462,8 @@ void  Pos_PrintOnRegion(struct PostQuantity      *NCPQ_P,
         if (PSO_P->SendToServer && strcmp(PSO_P->SendToServer, "No")){
           std::vector<double> v;
           Export_Value(&Value, v, PSO_P->SendToServerList);
-          Message::AddOnelabNumberChoice(PSO_P->SendToServer, v, PSO_P->Color);
+          Message::AddOnelabNumberChoice(PSO_P->SendToServer, v, PSO_P->Color,
+                                         PSO_P->Units, PSO_P->Label);
         }
       }
 
@@ -1495,7 +1496,8 @@ void  Pos_PrintOnRegion(struct PostQuantity      *NCPQ_P,
       if (PSO_P->SendToServer && strcmp(PSO_P->SendToServer, "No")){
         std::vector<double> v;
         Export_Value(&ValueSummed, v, PSO_P->SendToServerList);
-        Message::AddOnelabNumberChoice(PSO_P->SendToServer, v, PSO_P->Color);
+        Message::AddOnelabNumberChoice(PSO_P->SendToServer, v, PSO_P->Color,
+                                       PSO_P->Units, PSO_P->Label);
       }
     }
   }
