@@ -2,7 +2,7 @@
 
 SRC=${HOME}/src/getdp/benchmarks
 DST=/onelab_files
-MODELS='machines relay inductor indheat magnetometer antennas acoustic_scattering time_reversal shielding waveguides transfo_simple ddm_waves bloch_periodic_waveguides magnets thermal_conduction magnetostriction superconductors'
+MODELS='machines relay inductor indheat magnetometer antennas acoustic_scattering time_reversal shielding waveguides transfo_simple ddm_waves bloch_periodic_waveguides magnets thermal_conduction magnetostriction superconductors diffraction_grating'
 
 # sync files from local svn checkout
 for m in ${MODELS}; do
@@ -21,4 +21,5 @@ sudo rm -f ${DST}/templates.zip
 cd ${DST} && sudo zip -r ${DST}/templates.zip templates -x \*.svn\* -x \*.png
 
 # fix permissions
-sudo chown -R onelab:onelab ${DST}/
+#sudo chown -R onelab:onelab ${DST}/
+sudo chown -R geuzaine:geuzaine ${DST}/
